@@ -273,7 +273,7 @@ cdparanoia_class_init (CDParanoiaClass * klass)
           "Type of checking to perform", GST_TYPE_PARANOIA_MODE, 0,
           G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_DISCID,
-      g_param_spec_string ("discid", "discid", "The dics id", NULL,
+      g_param_spec_string ("discid", "discid", "The disc id", NULL,
           G_PARAM_READABLE));
 
   /* tags */
@@ -572,13 +572,22 @@ cdparanoia_get (GstPad * pad)
 
     if (!src->discont_sent && (is_track_switch (src, src->cur_sector) ||
             (src->prev_sec != src->cur_sector))) {
+      GstEvent *discont_ev;
+
+      if (src->cur_track == src->d->tracks) {
+        GST_DEBUG_OBJECT (src, "End of CD");
+        discont_ev = gst_event_new (GST_EVENT_EOS);
+        gst_element_set_eos (GST_ELEMENT (src));
+        return GST_DATA (discont_ev);
+      }
+
       gst_pad_convert (src->srcpad, sector_format,
           get_relative (src, src->cur_track, src->cur_sector), &format,
           &timestamp);
-      GstEvent *discont_ev;
 
       if (src->flush_pending) {
         src->flush_pending = FALSE;
+        GST_DEBUG_OBJECT (src, "Sending flush");
         return GST_DATA (gst_event_new (GST_EVENT_FLUSH));
       }
 
@@ -587,6 +596,8 @@ cdparanoia_get (GstPad * pad)
           GST_FORMAT_UNDEFINED);
 
       src->discont_sent = TRUE;
+      GST_DEBUG_OBJECT (src, "Sending discont to %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (timestamp));
       return GST_DATA (discont_ev);
     }
 
@@ -789,8 +800,7 @@ cdparanoia_open (CDParanoia * src)
   /* fail if the device couldn't be found */
   if (src->d == NULL) {
     GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
-        (_("Could not open CD device %s for reading."),
-            src->d->cdda_device_name), ("cdda_identify failed"));
+        (_("Could not open CD device for reading.")), ("cdda_identify failed"));
     return FALSE;
   }
 
@@ -841,13 +851,15 @@ cdparanoia_open (CDParanoia * src)
     cdda_speed_set (src->d, src->read_speed);
   }
 
-  /* save thse ones - skip lead-in */
+  /* save these ones - skip lead-in in
+   * in track mode, we need last_sector set to play the last track.
+   */
+  src->last_sector = cdda_disc_lastsector (src->d);
   if (src->track_mode) {
     src->first_sector = get_first_sector (src, src->cur_track);
     src->last_sector = get_first_sector (src, src->cur_track + 1) - 1;
   } else {
     src->first_sector = cdda_track_firstsector (src->d, 1);
-    src->last_sector = cdda_disc_lastsector (src->d);
   }
 
   /* this is the default segment we will play */
@@ -932,6 +944,7 @@ cdparanoia_change_state (GstElement * element)
         gst_event_unref (cdparanoia->pending_metadata);
         cdparanoia->pending_metadata = NULL;
       }
+      cdparanoia->track_mode = FALSE;
       break;
     case GST_STATE_READY_TO_NULL:
       cdparanoia_close (CDPARANOIA (element));
@@ -1014,9 +1027,11 @@ cdparanoia_event (GstPad * pad, GstEvent * event)
 
         set_track (src, track);
         seg_start_sector = get_first_sector (src, track);
-        seg_end_sector = get_first_sector (src, endtrack);
-        GST_LOG ("Seeking to track %d (sector %" G_GINT64_FORMAT ")",
-            track, seg_start_sector);
+        if (endoffset != -1)
+          seg_end_sector = get_first_sector (src, endtrack);
+        GST_LOG ("Seeking to track %d-%d (sector %" G_GINT64_FORMAT "-%"
+            G_GINT64_FORMAT ")", track, endtrack, seg_start_sector,
+            seg_end_sector);
       } else {
         /* from here on, all seeks are relative to the current track */
 
@@ -1106,7 +1121,7 @@ cdparanoia_event (GstPad * pad, GstEvent * event)
         src->segment_end_sector = seg_end_sector;
       }
       src->flush_pending = GST_EVENT_SEEK_FLAGS (event) & GST_SEEK_FLAG_FLUSH;
-      GST_DEBUG ("configured for %d -> %d sectors\n",
+      GST_DEBUG ("configured for %d -> %d sectors",
           src->segment_start_sector, src->segment_end_sector);
       break;
     }
