@@ -73,14 +73,15 @@ static GstStaticPadTemplate gst_gdk_pixbuf_sink_template =
         "image/bmp; "
         "image/x-bmp; "
         "image/x-MS-bmp; "
-        "image/vnd.wap.wbmp; " "image/x-bitmap; " "image/x-tga")
+        "image/vnd.wap.wbmp; " "image/x-bitmap; " "image/x-tga; "
+        "image/x-pcx; image/svg; image/svg+xml")
     );
 
 static GstStaticPadTemplate gst_gdk_pixbuf_src_template =
-GST_STATIC_PAD_TEMPLATE ("src",
+    GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_RGB)
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_RGB "; " GST_VIDEO_CAPS_RGBA)
     );
 
 gboolean pixbufscale_init (GstPlugin * plugin);
@@ -139,6 +140,7 @@ gst_gdk_pixbuf_get_capslist (void)
   char **mimetypes;
   char **mimetype;
   GstCaps *capslist = NULL;
+  GstCaps *return_caps = NULL;
 
   capslist = gst_caps_new_empty ();
   slist0 = gdk_pixbuf_get_formats ();
@@ -153,7 +155,11 @@ gst_gdk_pixbuf_get_capslist (void)
   }
   g_slist_free (slist0);
 
-  return capslist;
+  return_caps = gst_caps_intersect (capslist,
+      gst_static_caps_get (&gst_gdk_pixbuf_sink_template.static_caps));
+
+  gst_caps_free (capslist);
+  return return_caps;
 }
 #endif
 
@@ -292,26 +298,39 @@ gst_gdk_pixbuf_chain (GstPad * pad, GstData * _data)
       GError *error = NULL;
 
       if (!gdk_pixbuf_loader_close (filter->pixbuf_loader, &error)) {
-        GST_ELEMENT_ERROR (filter, LIBRARY, SHUTDOWN, (NULL), (error->message));
+        GST_ERROR_OBJECT (filter, error->message);
         g_error_free (error);
-        return;
       }
 
-      pixbuf = gdk_pixbuf_loader_get_pixbuf (filter->pixbuf_loader);
+      if (!(pixbuf = gdk_pixbuf_loader_get_pixbuf (filter->pixbuf_loader)))
+        goto next;
 
       if (filter->image_size == 0) {
         GstCaps *caps;
 
         filter->width = gdk_pixbuf_get_width (pixbuf);
         filter->height = gdk_pixbuf_get_height (pixbuf);
+        /* gdk_pixbuf likes to pad rowstride to 4 byte boundaries which we can't do
+         * at the moment
+         */
         filter->rowstride = gdk_pixbuf_get_rowstride (pixbuf);
         filter->image_size = filter->rowstride * filter->height;
 
-        caps = gst_caps_copy (gst_pad_get_pad_template_caps (filter->srcpad));
+        if (gdk_pixbuf_get_rowstride (pixbuf) / filter->width == 4) {
+          caps = gst_caps_from_string (GST_VIDEO_CAPS_RGBA);
+        } else if (gdk_pixbuf_get_rowstride (pixbuf) / filter->width == 3) {
+          caps = gst_caps_from_string (GST_VIDEO_CAPS_RGB);
+        } else {
+          GST_ELEMENT_ERROR (filter, CORE, NEGOTIATION, (NULL),
+              ("Bpp %d not supported",
+                  gdk_pixbuf_get_bits_per_sample (pixbuf)));
+          return;
+        }
         gst_caps_set_simple (caps,
             "width", G_TYPE_INT, filter->width,
             "height", G_TYPE_INT, filter->height,
             "framerate", G_TYPE_DOUBLE, filter->framerate, NULL);
+        GST_DEBUG ("Set size to %dx%d", filter->width, filter->height);
 
         gst_pad_set_explicit_caps (filter->srcpad, caps);
       }
@@ -321,9 +340,24 @@ gst_gdk_pixbuf_chain (GstPad * pad, GstData * _data)
       GST_BUFFER_TIMESTAMP (outbuf) = GST_BUFFER_TIMESTAMP (buf);
       GST_BUFFER_DURATION (outbuf) = GST_BUFFER_DURATION (buf);
 
-      memcpy (GST_BUFFER_DATA (outbuf), gdk_pixbuf_get_pixels (pixbuf),
-          filter->image_size);
+      {
+        int y;
+        guint8 *out_pix;
+        guint8 *in_pix;
+        int in_rowstride;
 
+        in_pix = gdk_pixbuf_get_pixels (pixbuf);
+        in_rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+        out_pix = GST_BUFFER_DATA (outbuf);
+
+        for (y = 0; y < filter->height; y++) {
+          memcpy (out_pix, in_pix, filter->rowstride);
+          in_pix += in_rowstride;
+          out_pix += filter->rowstride;
+        }
+      }
+
+      GST_DEBUG ("pushing...");
       gst_pad_push (filter->srcpad, GST_DATA (outbuf));
 
       g_object_unref (G_OBJECT (filter->pixbuf_loader));
@@ -339,7 +373,7 @@ gst_gdk_pixbuf_chain (GstPad * pad, GstData * _data)
       filter->pixbuf_loader = NULL;
     }
   }
-
+next:
   if (GST_IS_BUFFER (_data)) {
     if (filter->pixbuf_loader == NULL) {
       filter->pixbuf_loader = gdk_pixbuf_loader_new ();
