@@ -23,32 +23,36 @@
 #  include "config.h"
 #endif
 
+#include "gst/gst-i18n-plugin.h"
+
 #include "gstgnomevfs.h"
+#include <libgnomevfs/gnome-vfs.h>
 #include <gst/gst.h>
 
-static gboolean 
-plugin_init(GstPlugin *plugin)
+GST_DEBUG_CATEGORY (gnomevfs_debug);
+
+static gboolean
+plugin_init (GstPlugin * plugin)
 {
-  if (!gst_element_register (plugin, "gnomevfssrc", 
-	       GST_RANK_SECONDARY, gst_gnomevfssrc_get_type()) ||
-      !gst_element_register (plugin, "gnomevfssink", 
-	       GST_RANK_SECONDARY, gst_gnomevfssink_get_type())) {
+  gnome_vfs_init ();
+
+  if (!gst_element_register (plugin, "gnomevfssrc",
+          GST_RANK_SECONDARY, gst_gnomevfssrc_get_type ()) ||
+      !gst_element_register (plugin, "gnomevfssink",
+          GST_RANK_SECONDARY, gst_gnomevfssink_get_type ())) {
     return FALSE;
   }
+#ifdef ENABLE_NLS
+  setlocale (LC_ALL, "");
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+#endif /* ENABLE_NLS */
 
+  GST_DEBUG_CATEGORY_INIT (gnomevfs_debug, "gnomevfs", 0, "gnome-vfs elements");
   return TRUE;
 }
 
-GST_PLUGIN_DEFINE (
-  GST_VERSION_MAJOR,
-  GST_VERSION_MINOR,
-  "gnomevfs",
-  "elements to access the Gnome vfs",
-  plugin_init,
-  VERSION,
-  GST_LICENSE,
-  GST_COPYRIGHT,
-  GST_PACKAGE,
-  GST_ORIGIN
-)
-
+GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
+    GST_VERSION_MINOR,
+    "gnomevfs",
+    "elements to access the Gnome vfs",
+    plugin_init, VERSION, GST_LICENSE, GST_PACKAGE, GST_ORIGIN)
