@@ -54,6 +54,9 @@
 /* gnome-vfs.h doesn't include the following header, which we need: */
 #include <libgnomevfs/gnome-vfs-standard-callbacks.h>
 
+GST_DEBUG_CATEGORY_EXTERN (gnomevfs_debug);
+#define GST_CAT_DEFAULT gnomevfs_debug
+
 
 #define GST_TYPE_GNOMEVFSSRC \
   (gst_gnomevfssrc_get_type())
@@ -109,6 +112,7 @@ struct _GstGnomeVFSSrc
   gboolean iradio_mode;
   gboolean http_callbacks_pushed;
   gboolean seekable;
+  GstTagList *icy_taglist;
 
   gint icy_metaint;
   GnomeVFSFileSize icy_count;
@@ -624,7 +628,7 @@ audiocast_init (GstGnomeVFSSrc * src)
 
   if (!src->iradio_mode)
     return TRUE;
-  GST_DEBUG ("audiocast: registering listener");
+  GST_DEBUG_OBJECT (src, "audiocast: registering listener");
   if (audiocast_register_listener (&src->audiocast_port,
           &src->audiocast_fd) < 0) {
     GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
@@ -632,7 +636,7 @@ audiocast_init (GstGnomeVFSSrc * src)
     close (src->audiocast_fd);
     return FALSE;
   }
-  GST_DEBUG ("audiocast: creating pipe");
+  GST_DEBUG_OBJECT (src, "audiocast: creating pipe");
   src->audiocast_notify_queue = NULL;
   if (pipe (pipefds) < 0) {
     close (src->audiocast_fd);
@@ -640,7 +644,7 @@ audiocast_init (GstGnomeVFSSrc * src)
   }
   src->audiocast_thread_die_infd = pipefds[0];
   src->audiocast_thread_die_outfd = pipefds[1];
-  GST_DEBUG ("audiocast: creating audiocast thread");
+  GST_DEBUG_OBJECT (src, "audiocast: creating audiocast thread");
   src->audiocast_thread =
       g_thread_create ((GThreadFunc) audiocast_thread_run, src, TRUE, &error);
   if (error != NULL) {
@@ -692,16 +696,40 @@ static void
 audiocast_do_notifications (GstGnomeVFSSrc * src)
 {
   /* Send any pending notifications we got from the UDP thread. */
-  if (src->iradio_mode) {
-    GList *entry;
+  GList *entry;
+  GstTagList *taglist;
+  gboolean set_taglist = FALSE;
 
-    g_mutex_lock (src->audiocast_queue_mutex);
-    for (entry = src->audiocast_notify_queue; entry; entry = entry->next)
-      g_object_notify (G_OBJECT (src), (const gchar *) entry->data);
-    g_list_free (src->audiocast_notify_queue);
-    src->audiocast_notify_queue = NULL;
-    g_mutex_unlock (src->audiocast_queue_mutex);
+  if (!src->iradio_mode)
+    return;
+
+  if (src->icy_taglist == NULL) {
+    taglist = gst_tag_list_new ();
+  } else {
+    taglist = src->icy_taglist;
+    set_taglist = TRUE;
   }
+
+  g_mutex_lock (src->audiocast_queue_mutex);
+  for (entry = src->audiocast_notify_queue; entry; entry = entry->next) {
+    g_object_notify (G_OBJECT (src), (const gchar *) entry->data);
+
+    if (strcmp (entry->data, "iradio-url") == 0) {
+      gst_tag_list_add (taglist, GST_TAG_MERGE_REPLACE, GST_TAG_LOCATION,
+          src->iradio_url, NULL);
+    } else if (strcmp (entry->data, "iradio-title") == 0) {
+      gst_tag_list_add (taglist, GST_TAG_MERGE_REPLACE, GST_TAG_TITLE,
+          src->iradio_title, NULL);
+    }
+  }
+  g_list_free (src->audiocast_notify_queue);
+  src->audiocast_notify_queue = NULL;
+  g_mutex_unlock (src->audiocast_queue_mutex);
+
+  if (set_taglist)
+    src->icy_taglist = taglist;
+  else
+    gst_tag_list_free (taglist);
 }
 
 static gpointer
@@ -719,22 +747,26 @@ audiocast_thread_run (GstGnomeVFSSrc * src)
   FD_SET (src->audiocast_thread_die_infd, &fdset);
 
   while (1) {
-    GST_DEBUG ("audiocast thread: dropping into select");
+    GST_DEBUG_OBJECT (src, "audiocast thread: dropping into select");
     readset = fdset;
     if (select (FD_SETSIZE, &readset, NULL, NULL, NULL) < 0) {
-      perror ("select");
+      GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
+          ("Unable to select"));
       return NULL;
     }
     if (FD_ISSET (src->audiocast_thread_die_infd, &readset)) {
       char buf[1];
 
-      GST_DEBUG ("audiocast thread: got die character");
-      read (src->audiocast_thread_die_infd, buf, 1);
+      GST_DEBUG_OBJECT (src, "audiocast thread: got die character");
+      if (read (src->audiocast_thread_die_infd, buf, 1) != 1) {
+        GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
+            ("Unable to read die character"));
+      }
       close (src->audiocast_thread_die_infd);
       close (src->audiocast_fd);
       return NULL;
     }
-    GST_DEBUG ("audiocast thread: reading data");
+    GST_DEBUG_OBJECT (src, "audiocast thread: reading data");
     len =
         recvfrom (src->audiocast_fd, buf, sizeof (buf) - 1, 0,
         (struct sockaddr *) &from, &fromlen);
@@ -780,7 +812,7 @@ audiocast_thread_run (GstGnomeVFSSrc * src)
           g_mutex_lock (src->audiocast_queue_mutex);
           src->audiocast_notify_queue =
               g_list_append (src->audiocast_notify_queue, "iradio-title");
-          GST_DEBUG ("audiocast title: %s\n", src->iradio_title);
+          GST_DEBUG_OBJECT (src, "audiocast title: %s\n", src->iradio_title);
           g_mutex_unlock (src->audiocast_queue_mutex);
         } else if (!strncmp (lines[i], "x-audiocast-streamurl", 21)) {
           g_mutex_lock (src->audiocast_udpdata_mutex);
@@ -791,7 +823,7 @@ audiocast_thread_run (GstGnomeVFSSrc * src)
           g_mutex_lock (src->audiocast_queue_mutex);
           src->audiocast_notify_queue =
               g_list_append (src->audiocast_notify_queue, "iradio-url");
-          GST_DEBUG ("audiocast url: %s\n", src->iradio_title);
+          GST_DEBUG_OBJECT (src, "audiocast url: %s\n", src->iradio_title);
           g_mutex_unlock (src->audiocast_queue_mutex);
         } else if (!strncmp (lines[i], "x-audiocast-udpseqnr", 20)) {
           gchar outbuf[120];
@@ -805,7 +837,7 @@ audiocast_thread_run (GstGnomeVFSSrc * src)
                 strerror (errno));
             continue;
           }
-          GST_DEBUG ("sent audiocast ack: %s\n", outbuf);
+          GST_DEBUG_OBJECT (src, "sent audiocast ack: %s\n", outbuf);
         }
       }
       g_strfreev (lines);
@@ -826,10 +858,13 @@ audiocast_thread_kill (GstGnomeVFSSrc * src)
      metadata, then we don't need the
      audiocast metadata too.
    */
-  GST_DEBUG ("audiocast: writing die character");
-  write (src->audiocast_thread_die_outfd, "q", 1);
+  GST_DEBUG_OBJECT (src, "audiocast: writing die character");
+  if (write (src->audiocast_thread_die_outfd, "q", 1) != 1) {
+    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_WRITE, (NULL),
+        ("Unable to write die character"));
+  }
   close (src->audiocast_thread_die_outfd);
-  GST_DEBUG ("audiocast: joining thread");
+  GST_DEBUG_OBJECT (src, "audiocast: joining thread");
   g_thread_join (src->audiocast_thread);
   src->audiocast_thread = NULL;
 }
@@ -844,7 +879,7 @@ gst_gnomevfssrc_send_additional_headers_callback (gconstpointer in,
 
   if (!src->iradio_mode)
     return;
-  GST_DEBUG ("sending headers\n");
+  GST_DEBUG_OBJECT (src, "sending headers\n");
 
   out_args->headers = g_list_append (out_args->headers,
       g_strdup ("icy-metadata:1\r\n"));
@@ -861,11 +896,20 @@ gst_gnomevfssrc_received_headers_callback (gconstpointer in,
   GstGnomeVFSSrc *src = GST_GNOMEVFSSRC (callback_data);
   GnomeVFSModuleCallbackReceivedHeadersIn *in_args =
       (GnomeVFSModuleCallbackReceivedHeadersIn *) in;
+  GstTagList *tags = NULL;
+  gboolean set_taglist = FALSE;
 
   /* This is only used for internet radio stuff right now */
   if (!src->iradio_mode)
     return;
 
+  if (src->icy_taglist) {
+    tags = src->icy_taglist;
+    set_taglist = TRUE;
+  } else {
+    tags = gst_tag_list_new ();
+  }
+
   for (i = in_args->headers; i; i = i->next) {
     char *data = (char *) i->data;
     char *key = data;
@@ -881,9 +925,12 @@ gst_gnomevfssrc_received_headers_callback (gconstpointer in,
 
     /* Icecast stuff */
     if (!strncmp (data, "icy-metaint:", 12)) {  /* ugh */
-      sscanf (data + 12, "%d", &icy_metaint);
+      if (sscanf (data + 12, "%d", &icy_metaint) != 1) {
+        GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
+            ("Could not parse icy-metaint header"));
+      }
       src->icy_metaint = icy_metaint;
-      GST_DEBUG ("got icy-metaint %d, killing audiocast thread",
+      GST_DEBUG_OBJECT (src, "got icy-metaint %d, killing audiocast thread",
           src->icy_metaint);
       audiocast_thread_kill (src);
       continue;
@@ -896,50 +943,67 @@ gst_gnomevfssrc_received_headers_callback (gconstpointer in,
     else
       continue;
 
-    GST_DEBUG ("key: %s", key);
+    GST_DEBUG_OBJECT (src, "key: %s", key);
     if (!strncmp (key, "name", 4)) {
       g_free (src->iradio_name);
       src->iradio_name = gst_gnomevfssrc_unicodify (value);
-      if (src->iradio_name)
+      if (src->iradio_name) {
         g_object_notify (G_OBJECT (src), "iradio-name");
+        gst_tag_list_add (tags, GST_TAG_MERGE_REPLACE, GST_TAG_COMMENT,
+            src->iradio_name, NULL);
+        set_taglist = TRUE;
+      }
     } else if (!strncmp (key, "genre", 5)) {
       g_free (src->iradio_genre);
       src->iradio_genre = gst_gnomevfssrc_unicodify (value);
-      if (src->iradio_genre)
+      if (src->iradio_genre) {
         g_object_notify (G_OBJECT (src), "iradio-genre");
+        gst_tag_list_add (tags, GST_TAG_MERGE_REPLACE, GST_TAG_GENRE,
+            src->iradio_genre, NULL);
+        set_taglist = TRUE;
+      }
     } else if (!strncmp (key, "url", 3)) {
       g_free (src->iradio_url);
       src->iradio_url = gst_gnomevfssrc_unicodify (value);
-      if (src->iradio_url)
+      if (src->iradio_url) {
         g_object_notify (G_OBJECT (src), "iradio-url");
+        gst_tag_list_add (tags, GST_TAG_MERGE_REPLACE, GST_TAG_LOCATION,
+            src->iradio_url, NULL);
+        set_taglist = TRUE;
+      }
     }
   }
+
+  if (set_taglist)
+    src->icy_taglist = tags;
+  else
+    gst_tag_list_free (tags);
 }
 
 static void
-gst_gnomevfssrc_push_callbacks (GstGnomeVFSSrc * gnomevfssrc)
+gst_gnomevfssrc_push_callbacks (GstGnomeVFSSrc * src)
 {
-  if (gnomevfssrc->http_callbacks_pushed)
+  if (src->http_callbacks_pushed)
     return;
 
-  GST_DEBUG ("pushing callbacks");
+  GST_DEBUG_OBJECT (src, "pushing callbacks");
   gnome_vfs_module_callback_push
       (GNOME_VFS_MODULE_CALLBACK_HTTP_SEND_ADDITIONAL_HEADERS,
-      gst_gnomevfssrc_send_additional_headers_callback, gnomevfssrc, NULL);
+      gst_gnomevfssrc_send_additional_headers_callback, src, NULL);
   gnome_vfs_module_callback_push
       (GNOME_VFS_MODULE_CALLBACK_HTTP_RECEIVED_HEADERS,
-      gst_gnomevfssrc_received_headers_callback, gnomevfssrc, NULL);
+      gst_gnomevfssrc_received_headers_callback, src, NULL);
 
-  gnomevfssrc->http_callbacks_pushed = TRUE;
+  src->http_callbacks_pushed = TRUE;
 }
 
 static void
-gst_gnomevfssrc_pop_callbacks (GstGnomeVFSSrc * gnomevfssrc)
+gst_gnomevfssrc_pop_callbacks (GstGnomeVFSSrc * src)
 {
-  if (!gnomevfssrc->http_callbacks_pushed)
+  if (!src->http_callbacks_pushed)
     return;
 
-  GST_DEBUG ("popping callbacks");
+  GST_DEBUG_OBJECT (src, "popping callbacks");
   gnome_vfs_module_callback_pop
       (GNOME_VFS_MODULE_CALLBACK_HTTP_SEND_ADDITIONAL_HEADERS);
   gnome_vfs_module_callback_pop
@@ -956,9 +1020,11 @@ gst_gnomevfssrc_get_icy_metadata (GstGnomeVFSSrc * src)
   guchar *data;
   guchar *pos;
   gchar **tags;
+  GstTagList *taglist;
   int i;
+  gboolean set_taglist = FALSE;
 
-  GST_DEBUG ("reading icecast metadata");
+  GST_DEBUG_OBJECT (src, "reading icecast metadata");
 
   while (length == 0) {
     res = gnome_vfs_read (src->handle, &foobyte, 1, &length);
@@ -988,14 +1054,23 @@ gst_gnomevfssrc_get_icy_metadata (GstGnomeVFSSrc * src)
 
   data[metadata_length] = 0;
   tags = g_strsplit ((gchar *) data, "';", 0);
+  if (src->icy_taglist) {
+    taglist = src->icy_taglist;
+    set_taglist = TRUE;
+  } else {
+    taglist = gst_tag_list_new ();
+  }
 
   for (i = 0; tags[i]; i++) {
     if (!g_ascii_strncasecmp (tags[i], "StreamTitle=", 12)) {
       g_free (src->iradio_title);
       src->iradio_title = gst_gnomevfssrc_unicodify (tags[i] + 13);
       if (src->iradio_title) {
-        GST_DEBUG ("sending notification on icecast title");
+        GST_DEBUG_OBJECT (src, "sending notification on icecast title");
         g_object_notify (G_OBJECT (src), "iradio-title");
+        gst_tag_list_add (taglist, GST_TAG_MERGE_REPLACE, GST_TAG_TITLE,
+            src->iradio_title, NULL);
+        set_taglist = TRUE;
       } else
         g_print ("Unable to convert icecast title \"%s\" to UTF-8!\n",
             tags[i] + 13);
@@ -1005,8 +1080,11 @@ gst_gnomevfssrc_get_icy_metadata (GstGnomeVFSSrc * src)
       g_free (src->iradio_url);
       src->iradio_url = gst_gnomevfssrc_unicodify (tags[i] + 11);
       if (src->iradio_url) {
-        GST_DEBUG ("sending notification on icecast url");
+        GST_DEBUG_OBJECT (src, "sending notification on icecast url");
         g_object_notify (G_OBJECT (src), "iradio-url");
+        gst_tag_list_add (taglist, GST_TAG_MERGE_REPLACE, GST_TAG_LOCATION,
+            src->iradio_url, NULL);
+        set_taglist = TRUE;
       } else
         g_print ("Unable to convert icecast url \"%s\" to UTF-8!\n",
             tags[i] + 11);
@@ -1014,6 +1092,10 @@ gst_gnomevfssrc_get_icy_metadata (GstGnomeVFSSrc * src)
   }
 
   g_strfreev (tags);
+  if (set_taglist)
+    src->icy_taglist = taglist;
+  else
+    gst_tag_list_free (taglist);
 }
 
 /* end of icecast/audiocast metadata extraction support code */
@@ -1038,33 +1120,47 @@ gst_gnomevfssrc_get (GstPad * pad)
   g_return_val_if_fail (GST_FLAG_IS_SET (src, GST_GNOMEVFSSRC_OPEN), NULL);
 
   /* deal with EOF state */
-  if ((src->curoffset >= src->size) && (src->size != 0)) {
+  if ((src->curoffset >= src->size) && (src->size != 0) && !src->new_seek) {
     gst_element_set_eos (GST_ELEMENT (src));
-    GST_DEBUG ("Returning EOS");
+    GST_DEBUG_OBJECT (src, "Returning EOS");
     return GST_DATA (gst_event_new (GST_EVENT_EOS));
   }
 
+  /* grab tags from audiocast thread */
+  audiocast_do_notifications (src);
+
+  /* deal with tag list */
+  if (src->icy_taglist) {
+    gst_element_found_tags (GST_ELEMENT (src), src->icy_taglist);
+    GstEvent *event = gst_event_new_tag (src->icy_taglist);
+
+    gst_object_ref (GST_OBJECT (src));
+    event->src = GST_OBJECT (src);
+    src->icy_taglist = NULL;
+    return GST_DATA (event);
+  }
+
   /* create the buffer */
   /* FIXME: should eventually use a bufferpool for this */
   buf = gst_buffer_new ();
   g_return_val_if_fail (buf, NULL);
 
-  audiocast_do_notifications (src);
-
   if (src->iradio_mode && src->icy_metaint > 0) {
     GST_BUFFER_DATA (buf) = g_malloc0 (src->icy_metaint);
     data = GST_BUFFER_DATA (buf);
     g_return_val_if_fail (GST_BUFFER_DATA (buf) != NULL, NULL);
 
     GST_BUFFER_SIZE (buf) = 0;
-    GST_DEBUG ("doing read: icy_count: %" G_GINT64_FORMAT, src->icy_count);
-    result = gnome_vfs_read (src->handle, data,
-        src->icy_metaint - src->icy_count, &readbytes);
+    GST_DEBUG_OBJECT (src, "doing read: icy_count: %" G_GINT64_FORMAT,
+        src->icy_count);
+    result =
+        gnome_vfs_read (src->handle, data, src->icy_metaint - src->icy_count,
+        &readbytes);
 
     /* EOS? */
     if (readbytes == 0) {
       gst_buffer_unref (buf);
-      GST_DEBUG ("Returning EOS");
+      GST_DEBUG_OBJECT (src, "Returning EOS");
       gst_element_set_eos (GST_ELEMENT (src));
       return GST_DATA (gst_event_new (GST_EVENT_EOS));
     }
@@ -1076,6 +1172,7 @@ gst_gnomevfssrc_get (GstPad * pad)
     src->curoffset += readbytes;
 
     if (src->icy_count == src->icy_metaint) {
+      /* we'll push the taglist through next time around */
       gst_gnomevfssrc_get_icy_metadata (src);
       src->icy_count = 0;
     }
@@ -1089,7 +1186,7 @@ gst_gnomevfssrc_get (GstPad * pad)
 
       src->need_flush = FALSE;
       gst_buffer_unref (buf);
-      GST_DEBUG ("gnomevfssrc sending flush");
+      GST_DEBUG_OBJECT (src, "gnomevfssrc sending flush");
       return GST_DATA (event);
     }
 
@@ -1097,9 +1194,9 @@ gst_gnomevfssrc_get (GstPad * pad)
       GstEvent *event;
 
       gst_buffer_unref (buf);
-      GST_DEBUG ("new seek %" G_GINT64_FORMAT, src->curoffset);
+      GST_DEBUG_OBJECT (src, "new seek %" G_GINT64_FORMAT, src->curoffset);
       src->new_seek = FALSE;
-      GST_DEBUG ("gnomevfssrc sending discont");
+      GST_DEBUG_OBJECT (src, "gnomevfssrc sending discont");
       event =
           gst_event_new_discontinuous (FALSE, GST_FORMAT_BYTES, src->curoffset,
           NULL);
@@ -1109,12 +1206,13 @@ gst_gnomevfssrc_get (GstPad * pad)
     result = gnome_vfs_read (src->handle, GST_BUFFER_DATA (buf),
         src->bytes_per_read, &readbytes);
 
-    GST_DEBUG ("read: %s, readbytes: %" G_GINT64_FORMAT " @ %" G_GINT64_FORMAT,
+    GST_DEBUG_OBJECT (src,
+        "read: %s, readbytes: %" G_GINT64_FORMAT " @ %" G_GINT64_FORMAT,
         gnome_vfs_result_to_string (result), readbytes, src->curoffset);
     /* deal with EOS */
     if (readbytes == 0) {
       gst_buffer_unref (buf);
-      GST_DEBUG ("Returning EOS");
+      GST_DEBUG_OBJECT (src, "Returning EOS");
       gst_element_set_eos (GST_ELEMENT (src));
       return GST_DATA (gst_event_new (GST_EVENT_EOS));
     }
@@ -1163,9 +1261,15 @@ gst_gnomevfssrc_open_file (GstGnomeVFSSrc * src)
       gst_gnomevfssrc_pop_callbacks (src);
       audiocast_thread_kill (src);
 
-      GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
-          (_("Could not open vfs file \"%s\" for reading."), filename),
-          (gnome_vfs_result_to_string (result)));
+      if (result == GNOME_VFS_ERROR_NOT_FOUND) {
+        GST_ELEMENT_ERROR (src, RESOURCE, NOT_FOUND,
+            (_("Could not open vfs file \"%s\" for reading."), filename),
+            (gnome_vfs_result_to_string (result)));
+      } else {
+        GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
+            (_("Could not open vfs file \"%s\" for reading."), filename),
+            (gnome_vfs_result_to_string (result)));
+      }
       g_free (filename);
       return FALSE;
     }
@@ -1185,13 +1289,12 @@ gst_gnomevfssrc_open_file (GstGnomeVFSSrc * src)
     if (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SIZE)
       src->size = info->size;
   } else
-    GST_DEBUG ("getting info failed: %s", gnome_vfs_result_to_string (result));
+    GST_DEBUG_OBJECT (src, "getting info failed: %s",
+        gnome_vfs_result_to_string (result));
 
   gnome_vfs_file_info_unref (info);
 
-  GST_DEBUG ("size %" G_GINT64_FORMAT, src->size);
-
-  audiocast_do_notifications (src);
+  GST_DEBUG_OBJECT (src, "size %" G_GINT64_FORMAT, src->size);
 
   if (gnome_vfs_seek (src->handle, GNOME_VFS_SEEK_CURRENT, 0)
       == GNOME_VFS_OK) {
@@ -1321,7 +1424,7 @@ gst_gnomevfssrc_srcpad_event (GstPad * pad, GstEvent * event)
 
       result = gnome_vfs_seek (src->handle,
           GNOME_VFS_SEEK_START, desired_offset);
-      GST_DEBUG ("new_seek to %" G_GINT64_FORMAT ": %s",
+      GST_DEBUG_OBJECT (src, "new_seek to %" G_GINT64_FORMAT ": %s",
           desired_offset, gnome_vfs_result_to_string (result));
 
       if (result != GNOME_VFS_OK) {
