@@ -47,7 +47,7 @@ typedef enum
   GST_HERMES_COLORSPACE_YUY2_I420,
   GST_HERMES_COLORSPACE_RGB32_I420,
   GST_HERMES_COLORSPACE_RGB32_YV12,
-  GST_HERMES_COLORSPACE_420_SWAP,
+  GST_HERMES_COLORSPACE_420_SWAP
 }
 GstColorSpaceConverterType;
 
@@ -119,7 +119,7 @@ enum
 {
   ARG_0,
   ARG_SOURCE,
-  ARG_DEST,
+  ARG_DEST
 };
 
 static void gst_hermes_colorspace_base_init (gpointer g_class);
@@ -307,7 +307,6 @@ gst_hermes_colorspace_caps_remove_format_info (GstCaps * caps)
 {
   int i;
   GstStructure *structure;
-  GstCaps *rgbcaps;
 
   for (i = 0; i < gst_caps_get_size (caps); i++) {
     structure = gst_caps_get_structure (caps, i);
@@ -321,19 +320,18 @@ gst_hermes_colorspace_caps_remove_format_info (GstCaps * caps)
     gst_structure_remove_field (structure, "blue_mask");
   }
 
-  rgbcaps = gst_caps_simplify (caps);
-  gst_caps_free (caps);
+  gst_caps_do_simplify (caps);
 
-  return rgbcaps;
+  return caps;
 }
 
 static void
 gst_hermes_colorspace_structure_to_hermes_format (HermesFormat * format,
     GstStructure * structure)
 {
-  gst_structure_get_int (structure, "red_mask", &format->r);
-  gst_structure_get_int (structure, "green_mask", &format->g);
-  gst_structure_get_int (structure, "blue_mask", &format->b);
+  gst_structure_get_int (structure, "red_mask", (gint *) & format->r);
+  gst_structure_get_int (structure, "green_mask", (gint *) & format->g);
+  gst_structure_get_int (structure, "blue_mask", (gint *) & format->b);
   format->a = 0;
   gst_structure_get_int (structure, "bpp", &format->bits);
   format->indexed = 0;
