@@ -41,6 +41,8 @@ G_BEGIN_DECLS
 #define MPEG_TIME_TO_GST_TIME(time) (((time) * (GST_MSECOND/10)) / 9LL)
 #define GST_TIME_TO_MPEG_TIME(time) (((time) * 9LL) / (GST_MSECOND/10))
 
+#define GST_MPEG2DEC_NUM_BUFS 6
+
 typedef struct _GstMpeg2dec GstMpeg2dec;
 typedef struct _GstMpeg2decClass GstMpeg2decClass;
 
@@ -71,9 +73,12 @@ struct _GstMpeg2dec {
   gboolean	 closed;
   gboolean	 have_fbuf;
 
-  /* the timestamp of the next frame */
+  GstBuffer  *buffers[GST_MPEG2DEC_NUM_BUFS];
+
   DiscontState	 discont_state;
-  gint64	 next_time;
+
+  /* the timestamp of the next frame */
+  GstClockTime   next_time;
   gint64	 segment_start;
   gint64	 segment_end;
 
@@ -81,11 +86,15 @@ struct _GstMpeg2dec {
   Mpeg2decFormat format;
   gint		 width;
   gint		 height;
+  gint		 decoded_width;
+  gint		 decoded_height;
   gint		 pixel_width;
   gint		 pixel_height;
   gint		 frame_rate_code;
   gint64	 total_frames;
   gint64	 frame_period;
+  
+  guint64  offset;
   gdouble	 frame_rate;
   gboolean	 need_sequence;
 
@@ -93,6 +102,9 @@ struct _GstMpeg2dec {
 
   GstIndex	*index;
   gint		 index_id;
+
+  /* properties */
+  gint		 skip_frame;
 };
 
 struct _GstMpeg2decClass {
