@@ -270,7 +270,7 @@ vorbis_dec_src_event (GstPad * pad, GstEvent * event)
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:{
-      guint64 value;
+      gint64 value;
       GstFormat my_format = GST_FORMAT_DEFAULT;
 
       /* convert to granulepos */
@@ -298,7 +298,7 @@ vorbis_dec_src_event (GstPad * pad, GstEvent * event)
 static void
 vorbis_dec_event (GstVorbisDec * dec, GstEvent * event)
 {
-  guint64 value, time, bytes;
+  gint64 value, time, bytes;
 
   GST_LOG_OBJECT (dec, "handling event");
   switch (GST_EVENT_TYPE (event)) {
@@ -406,8 +406,8 @@ vorbis_dec_chain (GstPad * pad, GstData * data)
       gchar *encoder = NULL;
       gint bitrate = 0;
       GstTagList *list =
-          gst_tag_list_from_vorbiscomment_buffer (buf, "\003vorbis", 7,
-          &encoder);
+          gst_tag_list_from_vorbiscomment_buffer (buf, (guint8 *) "\003vorbis",
+          7, &encoder);
 
       if (!list) {
         GST_ERROR_OBJECT (vd, "couldn't decode comments");
