@@ -242,7 +242,7 @@ gst_audio_convert_fill_one_other (gfloat ** matrix,
     GstAudioChannelPosition to_pos_r,
     GstAudioChannelPosition to_pos_c, gfloat ratio)
 {
-  gfloat in_r, out_r[2];
+  gfloat in_r, out_r[2] = { 0, };
 
   /*
    * The idea is that we add up from the input (which means that if we
@@ -350,10 +350,10 @@ gst_audio_convert_fill_others (GstAudioConvert * this)
         GST_AUDIO_CHANNEL_POSITION_REAR_CENTER, RATIO_FRONT_REAR);
   } else if (in_has_center && !out_has_center && out_has_front) {
     gst_audio_convert_fill_one_other (this->matrix,
-        &this->sinkcaps, in_r,
-        GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
-        GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
-        GST_AUDIO_CHANNEL_POSITION_REAR_CENTER,
+        &this->sinkcaps, in_c,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT_OF_CENTER,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT_OF_CENTER,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
         &this->srccaps, out_f,
         GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
         GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
@@ -435,23 +435,25 @@ gst_audio_convert_fill_others (GstAudioConvert * this)
 }
 
 /*
- * Normalize output values.
+ * Normalize output values. The reason that we want to do it is related
+ * to the case where we convert 5.1 to 2.0; in this case, the surround
+ * channels will be mixed in the front channels, which makes their matrix
+ * value > 1.0, and thus increase loudness. We don't want that. However,
+ * for 2.0 to 5.1, we should not decrease volume, of course.
  */
 
 static void
 gst_audio_convert_fill_normalize (GstAudioConvert * this)
 {
-  gfloat sum, top = 0;
+  gfloat top = 1.0;
   gint i, j;
 
+  /* find max. */
   for (j = 0; j < this->srccaps.channels; j++) {
-    /* calculate sum */
-    sum = 0.0;
     for (i = 0; i < this->sinkcaps.channels; i++) {
-      sum += fabs (this->matrix[i][j]);
-    }
-    if (sum > top) {
-      top = sum;
+      if (this->matrix[i][j] > top) {
+        top = this->matrix[i][j];
+      }
     }
   }
 
