@@ -45,11 +45,13 @@ struct _GstAudiorate
 
   GstPad *sinkpad, *srcpad;
 
-  gint bytes_per_sample;
+  guint bytes_per_sample;
+  guint rate;
 
   /* audio state */
   guint64 next_offset;
 
+  gint64 start_offset;          /* -1 = wherever the input stream starts */
   guint64 in, out, add, drop;
   gboolean silent;
 };
@@ -66,14 +68,9 @@ GST_ELEMENT_DETAILS ("Audio rate adjuster",
     "Drops/duplicates/adjusts timestamps on audio samples to make a perfect stream",
     "Wim Taymans <wim@fluendo.com>");
 
-/* GstAudiorate signals and args */
-enum
-{
-  /* FILL ME */
-  LAST_SIGNAL
-};
-
+/* GstAudiorate args */
 #define DEFAULT_SILENT	TRUE
+#define DEFAULT_START_OFFSET 0
 
 enum
 {
@@ -83,7 +80,7 @@ enum
   ARG_ADD,
   ARG_DROP,
   ARG_SILENT,
-  /* FILL ME */
+  ARG_START_OFFSET
 };
 
 static GstStaticPadTemplate gst_audiorate_src_template =
@@ -114,8 +111,6 @@ static GstElementStateReturn gst_audiorate_change_state (GstElement * element);
 
 static GstElementClass *parent_class = NULL;
 
-/*static guint gst_audiorate_signals[LAST_SIGNAL] = { 0 }; */
-
 static GType
 gst_audiorate_get_type (void)
 {
@@ -180,6 +175,10 @@ gst_audiorate_class_init (GstAudiorateClass * klass)
       g_param_spec_boolean ("silent", "silent",
           "Don't emit notify for dropped and duplicated frames",
           DEFAULT_SILENT, G_PARAM_READWRITE));
+  g_object_class_install_property (object_class, ARG_START_OFFSET,
+      g_param_spec_int64 ("start-offset", "Start Offset",
+          "Start of stream in samples (-1 = wherever the input stream starts)",
+          -1, G_MAXINT64, DEFAULT_START_OFFSET, G_PARAM_READWRITE));
 
   element_class->change_state = gst_audiorate_change_state;
 }
@@ -191,7 +190,7 @@ gst_audiorate_link (GstPad * pad, const GstCaps * caps)
   GstStructure *structure;
   GstPad *otherpad;
   GstPadLinkReturn res;
-  gint ret, channels, depth;
+  gint channels, depth, rate;
 
   audiorate = GST_AUDIORATE (gst_pad_get_parent (pad));
 
@@ -204,9 +203,12 @@ gst_audiorate_link (GstPad * pad, const GstCaps * caps)
 
   structure = gst_caps_get_structure (caps, 0);
 
-  ret = gst_structure_get_int (structure, "channels", &channels);
-  ret &= gst_structure_get_int (structure, "depth", &depth);
+  if (!gst_structure_get_int (structure, "channels", &channels)
+      || !gst_structure_get_int (structure, "depth", &depth)
+      || !gst_structure_get_int (structure, "rate", &rate))
+    return GST_PAD_LINK_REFUSED;
 
+  audiorate->rate = rate;
   audiorate->bytes_per_sample = channels * (depth / 8);
   if (audiorate->bytes_per_sample == 0)
     audiorate->bytes_per_sample = 1;
@@ -236,6 +238,7 @@ gst_audiorate_init (GstAudiorate * audiorate)
   gst_pad_set_getcaps_function (audiorate->srcpad, gst_pad_proxy_getcaps);
 
   audiorate->bytes_per_sample = 1;
+  audiorate->rate = 0;
   audiorate->in = 0;
   audiorate->out = 0;
   audiorate->drop = 0;
@@ -243,6 +246,51 @@ gst_audiorate_init (GstAudiorate * audiorate)
   audiorate->silent = DEFAULT_SILENT;
 }
 
+static void
+gst_audiorate_chain_handle_event (GstAudiorate * audiorate, GstEvent * event)
+{
+  if (GST_EVENT_TYPE (event) == GST_EVENT_DISCONTINUOUS) {
+    /* If we have not received any data yet, start with whatever start
+     *   offset the input stream has, if this has been requested.  */
+    if (audiorate->in == 0 && audiorate->start_offset == -1) {
+      gint64 val;
+
+      if (gst_event_discont_get_value (event, GST_FORMAT_DEFAULT, &val)) {
+        audiorate->next_offset = val;
+        GST_LOG_OBJECT (audiorate, "discont event with offset = %"
+            G_GUINT64_FORMAT, val);
+      } else if (gst_event_discont_get_value (event, GST_FORMAT_TIME, &val)) {
+        audiorate->next_offset = val * audiorate->rate / GST_SECOND;
+        GST_LOG_OBJECT (audiorate, "discont event with time = %.6fs",
+            (double) val / GST_SECOND);
+      } else {
+        GST_WARNING_OBJECT (audiorate, "failed to handle discont event");
+      }
+      GST_LOG_OBJECT (audiorate, "stream starts at sample %" G_GUINT64_FORMAT,
+          audiorate->next_offset);
+    } else if (audiorate->in == 0 && audiorate->start_offset >= 0) {
+      audiorate->next_offset = audiorate->start_offset;
+      GST_LOG_OBJECT (audiorate, "stream starts at sample %" G_GUINT64_FORMAT,
+          audiorate->next_offset);
+    } else if (GST_EVENT_DISCONT_NEW_MEDIA (event)) {
+      /* pass on new media discont events, but fix the offset */
+      gst_data_unref (GST_DATA (event));
+      event = gst_event_new_discontinuous (TRUE, GST_FORMAT_DEFAULT,
+          audiorate->next_offset, GST_FORMAT_TIME,
+          audiorate->next_offset * GST_SECOND / audiorate->rate,
+          GST_FORMAT_UNDEFINED);
+      GST_LOG_OBJECT (audiorate, "new-media discont event");
+    } else {
+      /* drop mid-stream discont events, we will add or drop samples as needed */
+      gst_data_unref (GST_DATA (event));
+      GST_LOG_OBJECT (audiorate, "dropped mid-stream discont event");
+      return;
+    }
+  }
+
+  gst_pad_event_default (audiorate->sinkpad, event);
+}
+
 static void
 gst_audiorate_chain (GstPad * pad, GstData * data)
 {
@@ -250,18 +298,17 @@ gst_audiorate_chain (GstPad * pad, GstData * data)
   GstBuffer *buf;
   GstClockTime in_time, in_duration;
   guint64 in_offset, in_offset_end;
-  gint in_size;
+  gint in_size, in_samples;
 
   audiorate = GST_AUDIORATE (gst_pad_get_parent (pad));
 
   if (GST_IS_EVENT (data)) {
-    GstEvent *event = GST_EVENT (data);
-
-    gst_pad_event_default (pad, event);
+    gst_audiorate_chain_handle_event (audiorate, GST_EVENT (data));
     return;
   }
 
-  audiorate->in++;
+  g_return_if_fail (audiorate->rate > 0);
+  g_return_if_fail (audiorate->bytes_per_sample > 0);
 
   buf = GST_BUFFER (data);
   in_time = GST_BUFFER_TIMESTAMP (buf);
@@ -269,9 +316,42 @@ gst_audiorate_chain (GstPad * pad, GstData * data)
   in_size = GST_BUFFER_SIZE (buf);
   in_offset = GST_BUFFER_OFFSET (buf);
   in_offset_end = GST_BUFFER_OFFSET_END (buf);
+  in_samples = in_size / audiorate->bytes_per_sample;
 
-  if (in_offset == GST_CLOCK_TIME_NONE || in_offset_end == GST_CLOCK_TIME_NONE) {
+  audiorate->in += in_samples;
+
+  if (in_offset == GST_BUFFER_OFFSET_NONE && in_time != GST_CLOCK_TIME_NONE) {
+    in_offset = GST_BUFFER_OFFSET (buf) =
+        in_time * audiorate->rate / GST_SECOND;
+  }
+
+  if (in_offset_end == GST_BUFFER_OFFSET_NONE
+      && in_offset != GST_BUFFER_OFFSET_NONE && in_duration > 0
+      && in_duration != GST_CLOCK_TIME_NONE) {
+    /* would it be better to use in_samples here? */
+    in_offset_end = GST_BUFFER_OFFSET_END (buf) =
+        in_offset + (in_duration * audiorate->rate / GST_SECOND);
+  }
+
+  if (in_time == GST_CLOCK_TIME_NONE && in_offset != GST_BUFFER_OFFSET_NONE) {
+    in_time = GST_BUFFER_TIMESTAMP (buf) =
+        in_offset * GST_SECOND / audiorate->rate;
+  }
+
+  if ((in_duration == 0 || in_duration == GST_CLOCK_TIME_NONE)
+      && in_offset != GST_BUFFER_OFFSET_NONE
+      && in_offset_end != GST_BUFFER_OFFSET_NONE) {
+    in_duration = GST_BUFFER_DURATION (buf) =
+        (in_offset_end - in_offset) * GST_SECOND / audiorate->rate;
+  }
+
+  if (in_offset == GST_BUFFER_OFFSET_NONE
+      || in_offset_end == GST_BUFFER_OFFSET_NONE) {
     GST_WARNING_OBJECT (audiorate, "audiorate got buffer without offsets");
+    GST_ELEMENT_ERROR (audiorate, STREAM, DECODE, (NULL),
+        ("buffer without valid offsets or timestamp/duration"));
+    gst_data_unref (GST_DATA (buf));
+    return;
   }
 
   /* do we need to insert samples */
@@ -286,7 +366,8 @@ gst_audiorate_chain (GstPad * pad, GstData * data)
     fill = gst_buffer_new_and_alloc (fillsize);
     memset (GST_BUFFER_DATA (fill), 0, fillsize);
 
-    GST_LOG_OBJECT (audiorate, "inserting %lld samples", fillsamples);
+    GST_LOG_OBJECT (audiorate, "inserting %" G_GUINT64_FORMAT " samples",
+        fillsamples);
 
     GST_BUFFER_DURATION (fill) = in_duration * fillsize / in_size;
     GST_BUFFER_TIMESTAMP (fill) = in_time - GST_BUFFER_DURATION (fill);
@@ -294,7 +375,7 @@ gst_audiorate_chain (GstPad * pad, GstData * data)
     GST_BUFFER_OFFSET_END (fill) = in_offset;
 
     gst_pad_push (audiorate->srcpad, GST_DATA (fill));
-    audiorate->out++;
+    audiorate->out += fillsamples;
     audiorate->add += fillsamples;
 
     if (!audiorate->silent)
@@ -324,7 +405,7 @@ gst_audiorate_chain (GstPad * pad, GstData * data)
       truncsize = truncsamples * audiorate->bytes_per_sample;
       leftsize = in_size - truncsize;
 
-      trunc = gst_buffer_create_sub (buf, truncsize, in_size);
+      trunc = gst_buffer_create_sub (buf, truncsize, leftsize);
       GST_BUFFER_DURATION (trunc) = in_duration * leftsize / in_size;
       GST_BUFFER_TIMESTAMP (trunc) =
           in_time + in_duration - GST_BUFFER_DURATION (trunc);
@@ -339,8 +420,10 @@ gst_audiorate_chain (GstPad * pad, GstData * data)
       audiorate->drop += truncsamples;
     }
   }
+
+  audiorate->out += (GST_BUFFER_OFFSET_END (buf) - GST_BUFFER_OFFSET (buf));
+
   gst_pad_push (audiorate->srcpad, GST_DATA (buf));
-  audiorate->out++;
 
   audiorate->next_offset = in_offset_end;
 }
@@ -355,6 +438,9 @@ gst_audiorate_set_property (GObject * object,
     case ARG_SILENT:
       audiorate->silent = g_value_get_boolean (value);
       break;
+    case ARG_START_OFFSET:
+      audiorate->start_offset = g_value_get_int64 (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -383,6 +469,9 @@ gst_audiorate_get_property (GObject * object,
     case ARG_SILENT:
       g_value_set_boolean (value, audiorate->silent);
       break;
+    case ARG_START_OFFSET:
+      g_value_set_int64 (value, audiorate->start_offset);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -400,6 +489,12 @@ gst_audiorate_change_state (GstElement * element)
     case GST_STATE_READY_TO_PAUSED:
       audiorate->next_offset = 0;
       break;
+    case GST_STATE_NULL_TO_READY:
+      audiorate->in = 0;
+      audiorate->out = 0;
+      audiorate->drop = 0;
+      audiorate->add = 0;
+      break;
     default:
       break;
   }
