@@ -140,7 +140,7 @@ audioscale_get_type (void)
     };
 
     audioscale_type =
-        g_type_register_static (GST_TYPE_ELEMENT, "Audioscale",
+        g_type_register_static (GST_TYPE_ELEMENT, "GstAudioscale",
         &audioscale_info, 0);
   }
   return audioscale_type;
@@ -382,6 +382,8 @@ gst_audioscale_link (GstPad * pad, const GstCaps * caps)
     gst_resample_init (audioscale->gst_resample);
   }
 
+  audioscale->sample_duration = (GstClockTime) (GST_SECOND / r->o_rate);
+
   return link_ret;
 }
 
@@ -389,17 +391,28 @@ static void *
 gst_audioscale_get_buffer (void *priv, unsigned int size)
 {
   Audioscale *audioscale = priv;
+  gint64 samples;
+
+  GST_DEBUG ("size requested: %u irate: %f orate: %f offset %lld", size,
+      audioscale->gst_resample->i_rate, audioscale->gst_resample->o_rate,
+      audioscale->gst_resample_offset);
 
-  GST_DEBUG ("size requested: %u irate: %f orate: %f", size,
-      audioscale->gst_resample->i_rate, audioscale->gst_resample->o_rate);
   audioscale->outbuf = gst_buffer_new ();
   GST_BUFFER_SIZE (audioscale->outbuf) = size;
   GST_BUFFER_DATA (audioscale->outbuf) = g_malloc (size);
+  GST_BUFFER_OFFSET (audioscale->outbuf) = audioscale->gst_resample_offset;
+
   GST_BUFFER_TIMESTAMP (audioscale->outbuf) =
-      audioscale->gst_resample_offset * GST_SECOND /
-      audioscale->gst_resample->o_rate;
-  audioscale->gst_resample_offset +=
-      size / sizeof (gint16) / audioscale->gst_resample->channels;
+      audioscale->gst_resample_offset * audioscale->sample_duration;
+
+  samples =
+      GST_BUFFER_SIZE (audioscale->outbuf) / (sizeof (gint16) *
+      audioscale->gst_resample->channels);
+
+  GST_BUFFER_DURATION (audioscale->outbuf) =
+      samples * audioscale->sample_duration;
+
+  audioscale->gst_resample_offset += samples;
 
   return GST_BUFFER_DATA (audioscale->outbuf);
 }
@@ -434,8 +447,9 @@ gst_audioscale_decrease_rate (Audioscale * audioscale,
   }
 
   GST_BUFFER_DATA (outbuf) = (gpointer) outdata;
-  GST_BUFFER_TIMESTAMP (outbuf) =
-      audioscale->offsets[cur_iteration] * GST_SECOND / outrate;
+  GST_BUFFER_TIMESTAMP (outbuf) = GST_BUFFER_TIMESTAMP (buf);
+  GST_BUFFER_DURATION (outbuf) = GST_BUFFER_DURATION (buf);
+
   audioscale->offsets[cur_iteration] +=
       GST_BUFFER_SIZE (outbuf) / sizeof (gint16) /
       audioscale->gst_resample->channels;
@@ -473,8 +487,9 @@ gst_audioscale_increase_rate (Audioscale * audioscale,
   }
 
   GST_BUFFER_DATA (outbuf) = (gpointer) outdata;
-  GST_BUFFER_TIMESTAMP (outbuf) =
-      audioscale->offsets[cur_iteration] * GST_SECOND / outrate;
+  GST_BUFFER_TIMESTAMP (outbuf) = GST_BUFFER_TIMESTAMP (buf);
+  GST_BUFFER_DURATION (outbuf) = GST_BUFFER_DURATION (buf);
+
   audioscale->offsets[cur_iteration] +=
       GST_BUFFER_SIZE (outbuf) / sizeof (gint16) /
       audioscale->gst_resample->channels;
@@ -549,7 +564,6 @@ gst_audioscale_chain (GstPad * pad, GstData * _data)
 {
   GstBuffer *buf = GST_BUFFER (_data);
   GstBuffer *tempbuf, *tempbuf2;
-  GstClockTime outduration;
 
   Audioscale *audioscale;
   guchar *data;
@@ -563,45 +577,82 @@ gst_audioscale_chain (GstPad * pad, GstData * _data)
 
   audioscale = GST_AUDIOSCALE (gst_pad_get_parent (pad));
 
+  if (audioscale->gst_resample && audioscale->saved_discont) {
+    GstData *saved_event = GST_DATA (audioscale->saved_discont);
+
+    audioscale->saved_discont = NULL;
+    GST_DEBUG ("processing saved discont event");
+    gst_audioscale_chain (pad, saved_event);
+  }
+
   if (GST_IS_EVENT (_data)) {
     GstEvent *e = GST_EVENT (_data);
 
     switch (GST_EVENT_TYPE (e)) {
       case GST_EVENT_DISCONTINUOUS:{
-        gint64 new_off = 0;
+        gint64 ts = (gint64) GST_CLOCK_TIME_NONE;
+        gint64 off = (gint64) GST_BUFFER_OFFSET_NONE;
 
-        if (!audioscale->gst_resample) {
-          GST_LOG ("Discont before negotiation took place - ignoring");
-        } else if (gst_event_discont_get_value (e, GST_FORMAT_TIME, &new_off)) {
+        if (audioscale->passthru) {
+          gst_pad_event_default (pad, e);
+          return;
+        } else if (!audioscale->gst_resample) {
+          if (audioscale->saved_discont)
+            gst_data_unref (GST_DATA (audioscale->saved_discont));
+          audioscale->saved_discont = e;
+          GST_DEBUG ("Discont before negotiation happened, will process later");
+          return;               /* process it later when we're set up */
+        } else if (gst_event_discont_get_value (e, GST_FORMAT_TIME, &ts)) {
+          GST_DEBUG ("time discont [%" GST_TIME_FORMAT "]", GST_TIME_ARGS (ts));
           /* time -> out-sample */
-          new_off = new_off * audioscale->gst_resample->o_rate / GST_SECOND;
-        } else if (gst_event_discont_get_value (e,
-                GST_FORMAT_DEFAULT, &new_off)) {
+          off = ts / audioscale->sample_duration;
+          ts = off * audioscale->sample_duration;
+        } else if (gst_event_discont_get_value (e, GST_FORMAT_DEFAULT, &off)) {
+          GST_DEBUG ("default discont");
           /* in-sample -> out-sample */
-          new_off *= audioscale->gst_resample->o_rate;
-          new_off /= audioscale->gst_resample->i_rate;
-        } else if (gst_event_discont_get_value (e, GST_FORMAT_BYTES, &new_off)) {
-          new_off /= audioscale->gst_resample->channels;
-          new_off /=
-              (audioscale->gst_resample->format == GST_RESAMPLE_S16) ? 2 : 4;
-          new_off *= audioscale->gst_resample->o_rate;
-          new_off /= audioscale->gst_resample->i_rate;
+          off *= audioscale->gst_resample->o_rate;
+          off /= audioscale->gst_resample->i_rate;
+          ts = off * audioscale->sample_duration;
+        } else if (gst_event_discont_get_value (e, GST_FORMAT_BYTES, &off)) {
+          GST_DEBUG ("Discont event, bytes = %" G_GUINT64_FORMAT, off);
+          off /= audioscale->gst_resample->channels;
+          off /= (audioscale->gst_resample->format == GST_RESAMPLE_S16) ? 2 : 4;
+          off *= audioscale->gst_resample->o_rate;
+          off /= audioscale->gst_resample->i_rate;
+          ts = off * audioscale->sample_duration;
         } else {
           /* *sigh* */
           GST_DEBUG ("Discont without value - ignoring");
+          gst_data_unref (_data);
+          return;
         }
-        audioscale->gst_resample_offset = new_off;
-        /* fall-through */
+        audioscale->gst_resample_offset = off;
+
+        GST_DEBUG ("New resample offset after discont event: %"
+            G_GUINT64_FORMAT ", %" GST_TIME_FORMAT, off, GST_TIME_ARGS (ts));
+
+        e = gst_event_new_discontinuous (GST_EVENT_DISCONT_NEW_MEDIA (e),
+            GST_FORMAT_DEFAULT, off, GST_FORMAT_TIME, ts, GST_FORMAT_UNDEFINED);
+        gst_pad_event_default (pad, e);
+        gst_data_unref (_data);
+        return;
       }
+        break;
       default:
         gst_pad_event_default (pad, e);
+        return;
         break;
     }
-    return;
-  } else if (GST_BUFFER_TIMESTAMP_IS_VALID (buf) && audioscale->gst_resample) {
-    /* update time for out-sample */
-    audioscale->gst_resample_offset = GST_BUFFER_TIMESTAMP (buf) *
-        audioscale->gst_resample->o_rate / GST_SECOND;
+  }
+
+  if (GST_BUFFER_TIMESTAMP_IS_VALID (buf) && audioscale->gst_resample) {
+    GST_DEBUG ("input buffer timestamp [%" GST_TIME_FORMAT "] -> [%"
+        GST_TIME_FORMAT "] %d samples",
+        GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
+        GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf) + GST_BUFFER_DURATION (buf)),
+        GST_BUFFER_SIZE (buf) / (sizeof (gint16) *
+            audioscale->gst_resample->channels)
+        );
   }
 
   if (audioscale->passthru && audioscale->num_iterations == 0) {
@@ -611,7 +662,6 @@ gst_audioscale_chain (GstPad * pad, GstData * _data)
 
   data = GST_BUFFER_DATA (buf);
   size = GST_BUFFER_SIZE (buf);
-  outduration = GST_BUFFER_DURATION (buf);
 
   GST_DEBUG ("gst_audioscale_chain: got buffer of %ld bytes in '%s'\n",
       size, gst_element_get_name (GST_ELEMENT (audioscale)));
@@ -624,6 +674,7 @@ gst_audioscale_chain (GstPad * pad, GstData * _data)
     tempbuf = audioscale->outbuf;
     gst_buffer_unref (buf);
     outrate = audioscale->gst_resample->o_rate;
+
   }
   for (i = 0; i < audioscale->num_iterations; i++) {
     tempbuf2 = tempbuf;
@@ -649,7 +700,7 @@ gst_audioscale_chain (GstPad * pad, GstData * _data)
     gst_buffer_unref (tempbuf);
     tempbuf = audioscale->outbuf;
   }
-  GST_BUFFER_DURATION (tempbuf) = outduration;
+
   if (GST_BUFFER_SIZE (tempbuf) > 0)
     gst_pad_push (audioscale->srcpad, GST_DATA (tempbuf));
   else
