@@ -24,7 +24,7 @@
 #include "config.h"
 #endif
 #include <string.h>
-#include "gstsilence.h"
+#include <gstsilence.h>
 
 
 /* Silence signals and args */
