@@ -198,6 +198,8 @@ gst_client_status_get_type (void)
     {GST_CLIENT_STATUS_SLOW, "GST_CLIENT_STATUS_SLOW", "Too slow"},
     {GST_CLIENT_STATUS_ERROR, "GST_CLIENT_STATUS_ERROR", "Error"},
     {GST_CLIENT_STATUS_DUPLICATE, "GST_CLIENT_STATUS_DUPLICATE", "Duplicate"},
+    {GST_CLIENT_STATUS_NULLSTATE, "GST_CLIENT_STATUS_NULLSTATE",
+          "Please reconnect later"},
     {0, NULL, NULL},
   };
 
@@ -447,6 +449,17 @@ gst_multifdsink_add (GstMultiFdSink * sink, int fd)
 
   CLIENTS_LOCK (sink);
 
+  /* don't accept a client in NULL state */
+  if (GST_STATE (sink) == GST_STATE_NULL) {
+    client->status = GST_CLIENT_STATUS_NULLSTATE;
+    CLIENTS_UNLOCK (sink);
+    GST_WARNING_OBJECT (sink, "[fd %5d] element in NULL state, refusing", fd);
+    g_signal_emit (G_OBJECT (sink),
+        gst_multifdsink_signals[SIGNAL_CLIENT_REMOVED], 0, fd, client->status);
+    g_free (client);
+    return;
+  }
+
   /* check the hash to find a duplicate fd */
   clink = g_hash_table_lookup (sink->fd_hash, &client->fd.fd);
   if (clink != NULL) {
