@@ -45,9 +45,9 @@ enum
 {
   ARG_0,
   ARG_PORT,
-  ARG_CONTROL,
+  ARG_CONTROL
 /*  ARG_SOCKET_OPTIONS,*/
-  /* FILL ME */
+      /* FILL ME */
 };
 
 #define GST_TYPE_TCPSRC_CONTROL	(gst_tcpsrc_control_get_type())
@@ -60,6 +60,7 @@ gst_tcpsrc_control_get_type (void)
     {CONTROL_TCP, "2", "tcp"},
     {CONTROL_ZERO, NULL, NULL}
   };
+
   if (!tcpsrc_control_type) {
     tcpsrc_control_type =
         g_enum_register_static ("GstTCPSrcControl", tcpsrc_control);
@@ -103,6 +104,7 @@ gst_tcpsrc_get_type (void)
       (GInstanceInitFunc) gst_tcpsrc_init,
       NULL
     };
+
     tcpsrc_type =
         g_type_register_static (GST_TYPE_ELEMENT, "GstTCPSrc", &tcpsrc_info, 0);
   }
@@ -128,6 +130,9 @@ gst_tcpsrc_class_init (GstTCPSrc * klass)
 
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
+  gobject_class->set_property = gst_tcpsrc_set_property;
+  gobject_class->get_property = gst_tcpsrc_get_property;
+
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_PORT,
       g_param_spec_int ("port", "port", "The port to receive the packets from",
           0, 32768, TCP_DEFAULT_PORT, G_PARAM_READWRITE));
@@ -139,9 +144,6 @@ gst_tcpsrc_class_init (GstTCPSrc * klass)
     g_param_spec_boolean ("socketop", "socketop", "Enable or disable socket options REUSEADDR and KEEPALIVE",
                         FALSE, G_PARAM_READWRITE));
 */
-  gobject_class->set_property = gst_tcpsrc_set_property;
-  gobject_class->get_property = gst_tcpsrc_get_property;
-
   gstelement_class->change_state = gst_tcpsrc_change_state;
   gstelement_class->set_clock = gst_tcpsrc_set_clock;
 }
@@ -186,7 +188,10 @@ gst_tcpsrc_get (GstPad * pad)
   gint numbytes;
   fd_set read_fds;
   guint max_sock;
+
+#ifndef GST_DISABLE_LOADSAVE
   int ret, client_sock;
+#endif
   struct sockaddr client_addr;
 
   g_return_val_if_fail (pad != NULL, NULL);
@@ -215,8 +220,12 @@ gst_tcpsrc_get (GstPad * pad)
     if ((tcpsrc->control_sock != -1)
         && FD_ISSET (tcpsrc->control_sock, &read_fds)) {
       guchar *buf = NULL;
+
+#ifndef GST_DISABLE_LOADSAVE
       xmlDocPtr doc;
       GstCaps *caps;
+#endif
+
 
       switch (tcpsrc->control) {
         case CONTROL_TCP:
@@ -237,7 +246,7 @@ gst_tcpsrc_get (GstPad * pad)
 
           else {
             buf[ret] = '\0';
-            doc = xmlParseMemory (buf, ret);
+            doc = xmlParseMemory ((const gchar *) buf, ret);
             caps = gst_caps_load_thyself (doc->xmlRootNode);
 
             /* foward the connect, we don't signal back the result here... */
