@@ -480,7 +480,7 @@ gst_videofilter_setup (GstVideofilter * videofilter)
       videofilter->format->bpp) / 8;
 
   GST_DEBUG_OBJECT (videofilter, "from_buf_size %d to_buf_size %d",
-      videofilter->from_buf_size, videofilter->to_buf_size);
+    videofilter->from_buf_size, videofilter->to_buf_size);
   videofilter->inited = TRUE;
 }
 
@@ -567,8 +567,7 @@ gst_videofilter_class_add_pad_templates (GstVideofilterClass *
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  GST_DEBUG_CATEGORY_INIT (gst_videofilter_debug, "videofilter", 0,
-      "videofilter");
+  GST_DEBUG_CATEGORY_INIT (gst_videofilter_debug, "videofilter", 0, "videofilter");
   return TRUE;
 }
 
