@@ -32,8 +32,8 @@
 #include <string.h>
 #include <stdlib.h>
 
-static unsigned char
-random_char (void)
+static guint8
+random_guint8 (void)
 {
   static unsigned int state;
 
@@ -44,7 +44,7 @@ random_char (void)
 
 #if 0
 static void
-random_chars (unsigned char *dest, int nbytes)
+random_chars (guint8 * dest, int nbytes)
 {
   int i;
   static unsigned int state;
@@ -59,9 +59,9 @@ random_chars (unsigned char *dest, int nbytes)
 
 #if 0
 static void
-paint_rect_random (unsigned char *dest, int stride, int x, int y, int w, int h)
+paint_rect_random (guint8 * dest, int stride, int x, int y, int w, int h)
 {
-  unsigned char *d = dest + stride * y + x;
+  guint8 *d = dest + stride * y + x;
   int i;
 
   for (i = 0; i < h; i++) {
@@ -73,10 +73,9 @@ paint_rect_random (unsigned char *dest, int stride, int x, int y, int w, int h)
 
 #if 0
 static void
-paint_rect (unsigned char *dest, int stride, int x, int y, int w, int h,
-    unsigned char color)
+paint_rect (guint8 * dest, int stride, int x, int y, int w, int h, guint8 color)
 {
-  unsigned char *d = dest + stride * y + x;
+  guint8 *d = dest + stride * y + x;
   int i;
 
   for (i = 0; i < h; i++) {
@@ -88,11 +87,11 @@ paint_rect (unsigned char *dest, int stride, int x, int y, int w, int h,
 
 #if 0
 static void
-paint_rect_s2 (unsigned char *dest, int stride, int x, int y, int w, int h,
-    unsigned char col)
+paint_rect_s2 (guint8 * dest, int stride, int x, int y, int w, int h,
+    guint8 col)
 {
-  unsigned char *d = dest + stride * y + x * 2;
-  unsigned char *dp;
+  guint8 *d = dest + stride * y + x * 2;
+  guint8 *dp;
   int i, j;
 
   for (i = 0; i < h; i++) {
@@ -108,11 +107,11 @@ paint_rect_s2 (unsigned char *dest, int stride, int x, int y, int w, int h,
 
 #if 0
 static void
-paint_rect2 (unsigned char *dest, int stride, int x, int y, int w, int h,
-    unsigned char *col)
+paint_rect2 (guint8 * dest, int stride, int x, int y, int w, int h,
+    guint8 * col)
 {
-  unsigned char *d = dest + stride * y + x * 2;
-  unsigned char *dp;
+  guint8 *d = dest + stride * y + x * 2;
+  guint8 *dp;
   int i, j;
 
   for (i = 0; i < h; i++) {
@@ -128,11 +127,11 @@ paint_rect2 (unsigned char *dest, int stride, int x, int y, int w, int h,
 
 #if 0
 static void
-paint_rect3 (unsigned char *dest, int stride, int x, int y, int w, int h,
-    unsigned char *col)
+paint_rect3 (guint8 * dest, int stride, int x, int y, int w, int h,
+    guint8 * col)
 {
-  unsigned char *d = dest + stride * y + x * 3;
-  unsigned char *dp;
+  guint8 *d = dest + stride * y + x * 3;
+  guint8 *dp;
   int i, j;
 
   for (i = 0; i < h; i++) {
@@ -149,11 +148,11 @@ paint_rect3 (unsigned char *dest, int stride, int x, int y, int w, int h,
 
 #if 0
 static void
-paint_rect4 (unsigned char *dest, int stride, int x, int y, int w, int h,
-    unsigned char *col)
+paint_rect4 (guint8 * dest, int stride, int x, int y, int w, int h,
+    guint8 * col)
 {
-  unsigned char *d = dest + stride * y + x * 4;
-  unsigned char *dp;
+  guint8 *d = dest + stride * y + x * 4;
+  guint8 *dp;
   int i, j;
 
   for (i = 0; i < h; i++) {
@@ -171,11 +170,11 @@ paint_rect4 (unsigned char *dest, int stride, int x, int y, int w, int h,
 
 #if 0
 static void
-paint_rect_s4 (unsigned char *dest, int stride, int x, int y, int w, int h,
-    unsigned char col)
+paint_rect_s4 (guint8 * dest, int stride, int x, int y, int w, int h,
+    guint8 col)
 {
-  unsigned char *d = dest + stride * y + x * 4;
-  unsigned char *dp;
+  guint8 *d = dest + stride * y + x * 4;
+  guint8 *dp;
   int i, j;
 
   for (i = 0; i < h; i++) {
@@ -249,34 +248,38 @@ static int b_colors[] = { 255, 0, 255, 0, 255, 0, 255, 0, 128, 255, 0, 32 };
 #endif
 
 
-static void paint_setup_I420 (paintinfo * p, char *dest);
-static void paint_setup_YV12 (paintinfo * p, char *dest);
-static void paint_setup_YUY2 (paintinfo * p, char *dest);
-static void paint_setup_UYVY (paintinfo * p, char *dest);
-static void paint_setup_YVYU (paintinfo * p, char *dest);
-static void paint_setup_IYU2 (paintinfo * p, char *dest);
-static void paint_setup_Y800 (paintinfo * p, char *dest);
+static void paint_setup_I420 (paintinfo * p, guint8 * dest);
+static void paint_setup_YV12 (paintinfo * p, guint8 * dest);
+static void paint_setup_YUY2 (paintinfo * p, guint8 * dest);
+static void paint_setup_UYVY (paintinfo * p, guint8 * dest);
+static void paint_setup_YVYU (paintinfo * p, guint8 * dest);
+static void paint_setup_IYU2 (paintinfo * p, guint8 * dest);
+static void paint_setup_Y41B (paintinfo * p, guint8 * dest);
+static void paint_setup_Y42B (paintinfo * p, guint8 * dest);
+static void paint_setup_Y800 (paintinfo * p, guint8 * dest);
 
 #if 0
-static void paint_setup_IMC1 (paintinfo * p, char *dest);
-static void paint_setup_IMC2 (paintinfo * p, char *dest);
-static void paint_setup_IMC3 (paintinfo * p, char *dest);
-static void paint_setup_IMC4 (paintinfo * p, char *dest);
+static void paint_setup_IMC1 (paintinfo * p, guint8 * dest);
+static void paint_setup_IMC2 (paintinfo * p, guint8 * dest);
+static void paint_setup_IMC3 (paintinfo * p, guint8 * dest);
+static void paint_setup_IMC4 (paintinfo * p, guint8 * dest);
 #endif
-static void paint_setup_YUV9 (paintinfo * p, char *dest);
-static void paint_setup_YVU9 (paintinfo * p, char *dest);
-static void paint_setup_xRGB8888 (paintinfo * p, char *dest);
-static void paint_setup_xBGR8888 (paintinfo * p, char *dest);
-static void paint_setup_RGBx8888 (paintinfo * p, char *dest);
-static void paint_setup_BGRx8888 (paintinfo * p, char *dest);
-static void paint_setup_RGB888 (paintinfo * p, char *dest);
-static void paint_setup_BGR888 (paintinfo * p, char *dest);
-static void paint_setup_RGB565 (paintinfo * p, char *dest);
-static void paint_setup_xRGB1555 (paintinfo * p, char *dest);
+static void paint_setup_YUV9 (paintinfo * p, guint8 * dest);
+static void paint_setup_YVU9 (paintinfo * p, guint8 * dest);
+static void paint_setup_xRGB8888 (paintinfo * p, guint8 * dest);
+static void paint_setup_xBGR8888 (paintinfo * p, guint8 * dest);
+static void paint_setup_RGBx8888 (paintinfo * p, guint8 * dest);
+static void paint_setup_BGRx8888 (paintinfo * p, guint8 * dest);
+static void paint_setup_RGB888 (paintinfo * p, guint8 * dest);
+static void paint_setup_BGR888 (paintinfo * p, guint8 * dest);
+static void paint_setup_RGB565 (paintinfo * p, guint8 * dest);
+static void paint_setup_xRGB1555 (paintinfo * p, guint8 * dest);
 
 static void paint_hline_I420 (paintinfo * p, int x, int y, int w);
 static void paint_hline_YUY2 (paintinfo * p, int x, int y, int w);
 static void paint_hline_IYU2 (paintinfo * p, int x, int y, int w);
+static void paint_hline_Y41B (paintinfo * p, int x, int y, int w);
+static void paint_hline_Y42B (paintinfo * p, int x, int y, int w);
 static void paint_hline_Y800 (paintinfo * p, int x, int y, int w);
 
 #if 0
@@ -339,7 +342,9 @@ struct fourcc_list_struct fourcc_list[] = {
 #endif
   /* CLPL */
   /* Y41B */
+  {"Y41B", "Y41B", 12, paint_setup_Y41B, paint_hline_Y41B},
   /* Y42B */
+  {"Y42B", "Y42B", 16, paint_setup_Y42B, paint_hline_Y42B},
   /* Y800 grayscale */
   {"Y800", "Y800", 8, paint_setup_Y800, paint_hline_Y800},
 
@@ -372,7 +377,7 @@ paintinfo_find_by_structure (const GstStructure * structure)
   g_return_val_if_fail (structure, NULL);
 
   if (strcmp (media_type, "video/x-raw-yuv") == 0) {
-    char *s;
+    const char *s;
     int fourcc;
     guint32 format;
 
@@ -424,7 +429,7 @@ paintrect_find_fourcc (int find_fourcc)
   int i;
 
   for (i = 0; i < n_fourccs; i++) {
-    char *s;
+    const char *s;
     int fourcc;
 
     s = fourcc_list[i].fourcc;
@@ -508,7 +513,7 @@ gst_videotestsrc_get_size (GstVideotestsrc * v, int w, int h)
 }
 
 void
-gst_videotestsrc_smpte (GstVideotestsrc * v, unsigned char *dest, int w, int h)
+gst_videotestsrc_smpte (GstVideotestsrc * v, guint8 * dest, int w, int h)
 {
   int i;
   int y1, y2;
@@ -523,7 +528,7 @@ gst_videotestsrc_smpte (GstVideotestsrc * v, unsigned char *dest, int w, int h)
   if (fourcc == NULL)
     return;
 
-  fourcc->paint_setup (p, dest);
+  fourcc->paint_setup (p, (guint8 *) dest);
   p->paint_hline = fourcc->paint_hline;
 
   y1 = 2 * h / 3;
@@ -578,8 +583,8 @@ gst_videotestsrc_smpte (GstVideotestsrc * v, unsigned char *dest, int w, int h)
 
   /* superblack, black, dark grey */
   for (i = 0; i < 3; i++) {
-    int x1 = w / 2 + i * w / 12;
-    int x2 = w / 2 + (i + 1) * w / 12;
+    int x1 = (w + i * w / 6) / 2;
+    int x2 = (w + (i + 1) * w / 6) / 2;
     int k;
 
     if (i == 0) {
@@ -605,7 +610,7 @@ gst_videotestsrc_smpte (GstVideotestsrc * v, unsigned char *dest, int w, int h)
     for (i = x1; i < w; i++) {
       for (j = y2; j < h; j++) {
         /* FIXME not strictly correct */
-        color.Y = random_char ();
+        color.Y = random_guint8 ();
         color.R = color.Y;
         color.G = color.Y;
         color.B = color.Y;
@@ -617,7 +622,7 @@ gst_videotestsrc_smpte (GstVideotestsrc * v, unsigned char *dest, int w, int h)
 }
 
 void
-gst_videotestsrc_snow (GstVideotestsrc * v, unsigned char *dest, int w, int h)
+gst_videotestsrc_snow (GstVideotestsrc * v, guint8 * dest, int w, int h)
 {
   int i;
   int j;
@@ -632,7 +637,7 @@ gst_videotestsrc_snow (GstVideotestsrc * v, unsigned char *dest, int w, int h)
   if (fourcc == NULL)
     return;
 
-  fourcc->paint_setup (p, dest);
+  fourcc->paint_setup (p, (guint8 *) dest);
   p->paint_hline = fourcc->paint_hline;
 
   color = vts_colors[COLOR_BLACK];
@@ -641,7 +646,7 @@ gst_videotestsrc_snow (GstVideotestsrc * v, unsigned char *dest, int w, int h)
   for (i = 0; i < w; i++) {
     for (j = 0; j < h; j++) {
       /* FIXME not strictly correct */
-      color.Y = random_char ();
+      color.Y = random_guint8 ();
       color.R = color.Y;
       color.G = color.Y;
       color.B = color.Y;
@@ -651,7 +656,7 @@ gst_videotestsrc_snow (GstVideotestsrc * v, unsigned char *dest, int w, int h)
 }
 
 void
-gst_videotestsrc_black (GstVideotestsrc * v, unsigned char *dest, int w, int h)
+gst_videotestsrc_black (GstVideotestsrc * v, guint8 * dest, int w, int h)
 {
   int i;
   paintinfo pi;
@@ -664,7 +669,7 @@ gst_videotestsrc_black (GstVideotestsrc * v, unsigned char *dest, int w, int h)
   if (fourcc == NULL)
     return;
 
-  fourcc->paint_setup (p, dest);
+  fourcc->paint_setup (p, (guint8 *) dest);
   p->paint_hline = fourcc->paint_hline;
 
   p->color = vts_colors + COLOR_BLACK;
@@ -679,9 +684,9 @@ gst_videotestsrc_black (GstVideotestsrc * v, unsigned char *dest, int w, int h)
 #define ROUND_UP_8(x)  (((x)+7)&~7)
 
 static void
-paint_setup_I420 (paintinfo * p, char *dest)
+paint_setup_I420 (paintinfo * p, guint8 * dest)
 {
-  p->yp = dest;
+  p->yp = (guint8 *) dest;
   p->ystride = ROUND_UP_4 (p->width);
   p->up = p->yp + p->ystride * ROUND_UP_2 (p->height);
   p->ustride = ROUND_UP_8 (p->width) / 2;
@@ -694,7 +699,7 @@ static void
 paint_hline_I420 (paintinfo * p, int x, int y, int w)
 {
   int x1 = x / 2;
-  int x2 = (x + w) / 2;
+  int x2 = ROUND_UP_2 (x + w) / 2;
   int offset = y * p->ystride;
   int offset1 = (y / 2) * p->ustride;
 
@@ -704,9 +709,9 @@ paint_hline_I420 (paintinfo * p, int x, int y, int w)
 }
 
 static void
-paint_setup_YV12 (paintinfo * p, char *dest)
+paint_setup_YV12 (paintinfo * p, guint8 * dest)
 {
-  p->yp = dest;
+  p->yp = (guint8 *) dest;
   p->ystride = ROUND_UP_4 (p->width);
   p->vp = p->yp + p->ystride * ROUND_UP_2 (p->height);
   p->vstride = ROUND_UP_8 (p->ystride) / 2;
@@ -716,43 +721,43 @@ paint_setup_YV12 (paintinfo * p, char *dest)
 }
 
 static void
-paint_setup_YUY2 (paintinfo * p, char *dest)
+paint_setup_YUY2 (paintinfo * p, guint8 * dest)
 {
-  p->yp = dest;
-  p->up = dest + 1;
-  p->vp = dest + 3;
+  p->yp = (guint8 *) dest;
+  p->up = (guint8 *) dest + 1;
+  p->vp = (guint8 *) dest + 3;
   p->ystride = ROUND_UP_2 (p->width) * 2;
-  p->endptr = dest + p->ystride * p->height;
+  p->endptr = (guint8 *) dest + p->ystride * p->height;
 }
 
 static void
-paint_setup_UYVY (paintinfo * p, char *dest)
+paint_setup_UYVY (paintinfo * p, guint8 * dest)
 {
-  p->yp = dest + 1;
-  p->up = dest;
-  p->vp = dest + 2;
+  p->yp = (guint8 *) dest + 1;
+  p->up = (guint8 *) dest;
+  p->vp = (guint8 *) dest + 2;
   p->ystride = ROUND_UP_2 (p->width) * 2;
-  p->endptr = dest + p->ystride * p->height;
+  p->endptr = (guint8 *) dest + p->ystride * p->height;
 }
 
 static void
-paint_setup_YVYU (paintinfo * p, char *dest)
+paint_setup_YVYU (paintinfo * p, guint8 * dest)
 {
-  p->yp = dest;
-  p->up = dest + 3;
-  p->vp = dest + 1;
+  p->yp = (guint8 *) dest;
+  p->up = (guint8 *) dest + 3;
+  p->vp = (guint8 *) dest + 1;
   p->ystride = ROUND_UP_2 (p->width * 2);
-  p->endptr = dest + p->ystride * p->height;
+  p->endptr = (guint8 *) dest + p->ystride * p->height;
 }
 
 #ifndef HAVE_LIBOIL
 void
-splat_u8 (guint8 * dest, int dstr, guint8 val, int n)
+oil_splat_u8 (guint8 * dest, int dstr, guint8 * val, int n)
 {
   int i;
 
   for (i = 0; i < n; i++) {
-    *dest = val;
+    *dest = *val;
     dest += dstr;
   }
 }
@@ -764,42 +769,98 @@ paint_hline_YUY2 (paintinfo * p, int x, int y, int w)
   int x1 = x / 2;
   int x2 = (x + w) / 2;
   int offset;
+  guint8 Y = p->color->Y;
+  guint8 U = p->color->U;
+  guint8 V = p->color->V;
 
   offset = y * p->ystride;
-  splat_u8 (p->yp + offset + x * 2, 2, p->color->Y, w);
-  splat_u8 (p->up + offset + x1 * 4, 4, p->color->U, x2 - x1);
-  splat_u8 (p->vp + offset + x1 * 4, 4, p->color->V, x2 - x1);
+  oil_splat_u8 (p->yp + offset + x * 2, 2, &Y, w);
+  oil_splat_u8 (p->up + offset + x1 * 4, 4, &U, x2 - x1);
+  oil_splat_u8 (p->vp + offset + x1 * 4, 4, &V, x2 - x1);
 }
 
 static void
-paint_setup_IYU2 (paintinfo * p, char *dest)
+paint_setup_IYU2 (paintinfo * p, guint8 * dest)
 {
   /* untested */
-  p->yp = dest + 1;
-  p->up = dest + 0;
-  p->vp = dest + 2;
+  p->yp = (guint8 *) dest + 1;
+  p->up = (guint8 *) dest + 0;
+  p->vp = (guint8 *) dest + 2;
   p->ystride = ROUND_UP_4 (p->width * 3);
-  p->endptr = dest + p->ystride * p->height;
+  p->endptr = (guint8 *) dest + p->ystride * p->height;
 }
 
 static void
 paint_hline_IYU2 (paintinfo * p, int x, int y, int w)
 {
   int offset;
+  guint8 Y = p->color->Y;
+  guint8 U = p->color->U;
+  guint8 V = p->color->V;
 
   offset = y * p->ystride;
-  splat_u8 (p->yp + offset + x * 3, 3, p->color->Y, w);
-  splat_u8 (p->up + offset + x * 3, 3, p->color->U, w);
-  splat_u8 (p->vp + offset + x * 3, 3, p->color->V, w);
+  oil_splat_u8 (p->yp + offset + x * 3, 3, &Y, w);
+  oil_splat_u8 (p->up + offset + x * 3, 3, &U, w);
+  oil_splat_u8 (p->vp + offset + x * 3, 3, &V, w);
 }
 
 static void
-paint_setup_Y800 (paintinfo * p, char *dest)
+paint_setup_Y41B (paintinfo * p, guint8 * dest)
+{
+  p->yp = (guint8 *) dest;
+  p->ystride = ROUND_UP_4 (p->width);
+  p->up = p->yp + p->ystride * p->height;
+  p->ustride = ROUND_UP_8 (p->width) / 4;
+  p->vp = p->up + p->ustride * p->height;
+  p->vstride = ROUND_UP_8 (p->width) / 4;
+  p->endptr = p->vp + p->vstride * p->height;
+}
+
+static void
+paint_hline_Y41B (paintinfo * p, int x, int y, int w)
+{
+  int x1 = x / 4;
+  int x2 = (x + w) / 4;
+  int offset = y * p->ystride;
+  int offset1 = y * p->ustride;
+
+  memset (p->yp + offset + x, p->color->Y, w);
+  memset (p->up + offset1 + x1, p->color->U, x2 - x1);
+  memset (p->vp + offset1 + x1, p->color->V, x2 - x1);
+}
+
+static void
+paint_setup_Y42B (paintinfo * p, guint8 * dest)
+{
+  p->yp = (guint8 *) dest;
+  p->ystride = ROUND_UP_4 (p->width);
+  p->up = p->yp + p->ystride * p->height;
+  p->ustride = ROUND_UP_8 (p->width) / 2;
+  p->vp = p->up + p->ustride * p->height;
+  p->vstride = ROUND_UP_8 (p->width) / 2;
+  p->endptr = p->vp + p->vstride * p->height;
+}
+
+static void
+paint_hline_Y42B (paintinfo * p, int x, int y, int w)
+{
+  int x1 = x / 2;
+  int x2 = (x + w) / 2;
+  int offset = y * p->ystride;
+  int offset1 = y * p->ustride;
+
+  memset (p->yp + offset + x, p->color->Y, w);
+  memset (p->up + offset1 + x1, p->color->U, x2 - x1);
+  memset (p->vp + offset1 + x1, p->color->V, x2 - x1);
+}
+
+static void
+paint_setup_Y800 (paintinfo * p, guint8 * dest)
 {
   /* untested */
-  p->yp = dest;
+  p->yp = (guint8 *) dest;
   p->ystride = ROUND_UP_4 (p->width);
-  p->endptr = dest + p->ystride * p->height;
+  p->endptr = p->yp + p->ystride * p->height;
 }
 
 static void
@@ -812,7 +873,7 @@ paint_hline_Y800 (paintinfo * p, int x, int y, int w)
 
 #if 0
 static void
-paint_setup_IMC1 (paintinfo * p, char *dest)
+paint_setup_IMC1 (paintinfo * p, guint8 * dest)
 {
   p->yp = dest;
   p->up = dest + p->width * p->height;
@@ -820,7 +881,7 @@ paint_setup_IMC1 (paintinfo * p, char *dest)
 }
 
 static void
-paint_setup_IMC2 (paintinfo * p, char *dest)
+paint_setup_IMC2 (paintinfo * p, guint8 * dest)
 {
   p->yp = dest;
   p->vp = dest + p->width * p->height;
@@ -828,7 +889,7 @@ paint_setup_IMC2 (paintinfo * p, char *dest)
 }
 
 static void
-paint_setup_IMC3 (paintinfo * p, char *dest)
+paint_setup_IMC3 (paintinfo * p, guint8 * dest)
 {
   p->yp = dest;
   p->up = dest + p->width * p->height + p->width * p->height / 2;
@@ -836,7 +897,7 @@ paint_setup_IMC3 (paintinfo * p, char *dest)
 }
 
 static void
-paint_setup_IMC4 (paintinfo * p, char *dest)
+paint_setup_IMC4 (paintinfo * p, guint8 * dest)
 {
   p->yp = dest;
   p->vp = dest + p->width * p->height + p->width / 2;
@@ -858,11 +919,11 @@ paint_hline_IMC1 (paintinfo * p, int x, int y, int w)
 #endif
 
 static void
-paint_setup_YVU9 (paintinfo * p, char *dest)
+paint_setup_YVU9 (paintinfo * p, guint8 * dest)
 {
   int h = ROUND_UP_4 (p->height);
 
-  p->yp = dest;
+  p->yp = (guint8 *) dest;
   p->ystride = ROUND_UP_4 (p->width);
   p->vp = p->yp + p->ystride * ROUND_UP_4 (p->height);
   p->vstride = ROUND_UP_4 (p->ystride / 4);
@@ -872,12 +933,12 @@ paint_setup_YVU9 (paintinfo * p, char *dest)
 }
 
 static void
-paint_setup_YUV9 (paintinfo * p, char *dest)
+paint_setup_YUV9 (paintinfo * p, guint8 * dest)
 {
   /* untested */
   int h = ROUND_UP_4 (p->height);
 
-  p->yp = dest;
+  p->yp = (guint8 *) dest;
   p->ystride = ROUND_UP_4 (p->width);
   p->up = p->yp + p->ystride * h;
   p->ustride = ROUND_UP_4 (p->ystride / 4);
@@ -900,133 +961,139 @@ paint_hline_YUV9 (paintinfo * p, int x, int y, int w)
 }
 
 static void
-paint_setup_xRGB8888 (paintinfo * p, char *dest)
+paint_setup_xRGB8888 (paintinfo * p, guint8 * dest)
 {
-  p->yp = dest + 1;
-  p->up = dest + 2;
-  p->vp = dest + 3;
+  p->yp = (guint8 *) dest + 1;
+  p->up = (guint8 *) dest + 2;
+  p->vp = (guint8 *) dest + 3;
   p->ystride = p->width * 4;
-  p->endptr = p->dest + p->ystride * p->height;
+  p->endptr = dest + p->ystride * p->height;
 }
 
 static void
-paint_setup_xBGR8888 (paintinfo * p, char *dest)
+paint_setup_xBGR8888 (paintinfo * p, guint8 * dest)
 {
-  p->yp = dest + 3;
-  p->up = dest + 2;
-  p->vp = dest + 1;
+  p->yp = (guint8 *) dest + 3;
+  p->up = (guint8 *) dest + 2;
+  p->vp = (guint8 *) dest + 1;
   p->ystride = p->width * 4;
-  p->endptr = p->dest + p->ystride * p->height;
+  p->endptr = dest + p->ystride * p->height;
 }
 
 static void
-paint_setup_RGBx8888 (paintinfo * p, char *dest)
+paint_setup_RGBx8888 (paintinfo * p, guint8 * dest)
 {
-  p->yp = dest + 0;
-  p->up = dest + 1;
-  p->vp = dest + 2;
+  p->yp = (guint8 *) dest + 0;
+  p->up = (guint8 *) dest + 1;
+  p->vp = (guint8 *) dest + 2;
   p->ystride = p->width * 4;
-  p->endptr = p->dest + p->ystride * p->height;
+  p->endptr = dest + p->ystride * p->height;
 }
 
 static void
-paint_setup_BGRx8888 (paintinfo * p, char *dest)
+paint_setup_BGRx8888 (paintinfo * p, guint8 * dest)
 {
-  p->yp = dest + 2;
-  p->up = dest + 1;
-  p->vp = dest + 0;
+  p->yp = (guint8 *) dest + 2;
+  p->up = (guint8 *) dest + 1;
+  p->vp = (guint8 *) dest + 0;
   p->ystride = p->width * 4;
-  p->endptr = p->dest + p->ystride * p->height;
+  p->endptr = dest + p->ystride * p->height;
 }
 
 static void
-paint_setup_RGB888 (paintinfo * p, char *dest)
+paint_setup_RGB888 (paintinfo * p, guint8 * dest)
 {
-  p->yp = dest + 0;
-  p->up = dest + 1;
-  p->vp = dest + 2;
+  p->yp = (guint8 *) dest + 0;
+  p->up = (guint8 *) dest + 1;
+  p->vp = (guint8 *) dest + 2;
   p->ystride = ROUND_UP_4 (p->width * 3);
-  p->endptr = p->dest + p->ystride * p->height;
+  p->endptr = dest + p->ystride * p->height;
 }
 
 static void
-paint_setup_BGR888 (paintinfo * p, char *dest)
+paint_setup_BGR888 (paintinfo * p, guint8 * dest)
 {
-  p->yp = dest + 2;
-  p->up = dest + 1;
-  p->vp = dest + 0;
+  p->yp = (guint8 *) dest + 2;
+  p->up = (guint8 *) dest + 1;
+  p->vp = (guint8 *) dest + 0;
   p->ystride = ROUND_UP_4 (p->width * 3);
-  p->endptr = p->dest + p->ystride * p->height;
+  p->endptr = dest + p->ystride * p->height;
 }
 
 static void
 paint_hline_str4 (paintinfo * p, int x, int y, int w)
 {
   int offset = y * p->ystride;
+  guint8 R = p->color->R;
+  guint8 G = p->color->G;
+  guint8 B = p->color->B;
 
-  splat_u8 (p->yp + offset + x * 4, 4, p->color->R, w);
-  splat_u8 (p->up + offset + x * 4, 4, p->color->G, w);
-  splat_u8 (p->vp + offset + x * 4, 4, p->color->B, w);
+  oil_splat_u8 (p->yp + offset + x * 4, 4, &R, w);
+  oil_splat_u8 (p->up + offset + x * 4, 4, &G, w);
+  oil_splat_u8 (p->vp + offset + x * 4, 4, &B, w);
 }
 
 static void
 paint_hline_str3 (paintinfo * p, int x, int y, int w)
 {
   int offset = y * p->ystride;
+  guint8 R = p->color->R;
+  guint8 G = p->color->G;
+  guint8 B = p->color->B;
 
-  splat_u8 (p->yp + offset + x * 3, 3, p->color->R, w);
-  splat_u8 (p->up + offset + x * 3, 3, p->color->G, w);
-  splat_u8 (p->vp + offset + x * 3, 3, p->color->B, w);
+  oil_splat_u8 (p->yp + offset + x * 3, 3, &R, w);
+  oil_splat_u8 (p->up + offset + x * 3, 3, &G, w);
+  oil_splat_u8 (p->vp + offset + x * 3, 3, &B, w);
 }
 
 static void
-paint_setup_RGB565 (paintinfo * p, char *dest)
+paint_setup_RGB565 (paintinfo * p, guint8 * dest)
 {
-  p->yp = dest;
+  p->yp = (guint8 *) dest;
   p->ystride = ROUND_UP_4 (p->width * 2);
-  p->endptr = p->dest + p->ystride * p->height;
+  p->endptr = dest + p->ystride * p->height;
 }
 
 static void
 paint_hline_RGB565 (paintinfo * p, int x, int y, int w)
 {
   int offset = y * p->ystride;
-  unsigned int a, b;
+  guint8 a, b;
 
   a = (p->color->R & 0xf8) | (p->color->G >> 5);
   b = ((p->color->G << 3) & 0xe0) | (p->color->B >> 3);
 
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
-  splat_u8 (p->yp + offset + x * 2 + 0, 2, b, w);
-  splat_u8 (p->yp + offset + x * 2 + 1, 2, a, w);
+  oil_splat_u8 (p->yp + offset + x * 2 + 0, 2, &b, w);
+  oil_splat_u8 (p->yp + offset + x * 2 + 1, 2, &a, w);
 #else
-  splat_u8 (p->yp + offset + x * 2 + 0, 2, a, w);
-  splat_u8 (p->yp + offset + x * 2 + 1, 2, b, w);
+  oil_splat_u8 (p->yp + offset + x * 2 + 0, 2, &a, w);
+  oil_splat_u8 (p->yp + offset + x * 2 + 1, 2, &b, w);
 #endif
 }
 
 static void
-paint_setup_xRGB1555 (paintinfo * p, char *dest)
+paint_setup_xRGB1555 (paintinfo * p, guint8 * dest)
 {
-  p->yp = dest;
+  p->yp = (guint8 *) dest;
   p->ystride = ROUND_UP_4 (p->width * 2);
-  p->endptr = p->dest + p->ystride * p->height;
+  p->endptr = dest + p->ystride * p->height;
 }
 
 static void
 paint_hline_xRGB1555 (paintinfo * p, int x, int y, int w)
 {
   int offset = y * p->ystride;
-  unsigned int a, b;
+  guint8 a, b;
 
   a = ((p->color->R >> 1) & 0x7c) | (p->color->G >> 6);
   b = ((p->color->G << 2) & 0xe0) | (p->color->B >> 3);
 
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
-  splat_u8 (p->yp + offset + x * 2 + 0, 2, b, w);
-  splat_u8 (p->yp + offset + x * 2 + 1, 2, a, w);
+  oil_splat_u8 (p->yp + offset + x * 2 + 0, 2, &b, w);
+  oil_splat_u8 (p->yp + offset + x * 2 + 1, 2, &a, w);
 #else
-  splat_u8 (p->yp + offset + x * 2 + 0, 2, a, w);
-  splat_u8 (p->yp + offset + x * 2 + 1, 2, b, w);
+  oil_splat_u8 (p->yp + offset + x * 2 + 0, 2, &a, w);
+  oil_splat_u8 (p->yp + offset + x * 2 + 1, 2, &b, w);
 #endif
 }
